#    module      : transport.py
#
#    responsible : d033893
#
#    special area: SAP DB Loader
#
#    description : transport SAP DB databases via SAP DB Loader
#
#    prerequisite: Please install the loader Scripting Interface.
#
#    remarks:
#
#
#    ========== licence begin  GPL
#    Copyright (c) 2000-2004 SAP AG
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#    ========== licence end
#

#

#system
import sys
import os
import time
import socket
import thread
mutex = thread.allocate_lock()
#string
import string
import re
import regsub
#sax
from xml.sax            import saxlib
from xml.sax.drivers    import drv_xmlproc
#sapdb
import optlib
from   sapdb            import dbm,loader,sql

def _options ():
     return [
     #Mode
     ('demo', 'demo'                , ':', 'false'               , 'specifies demo   mode'  ),                     # TRUE , FALSE
     #Transformation Resources
     ('n', 'sourcenode'             , ':', socket.gethostname()  , 'specifies source servernode'),
     ('d', 'sourcedbname'           , ':', 'sourcedb'            , 'specifies source serverdb to create'),
     ('a', 'sourcedbauser'          , ':', 'DBA,DBA'             , 'specifies source SYSDBA user'),
     ('s', 'sourcedbmuser'          , ':', 'DBM,DBM'             , 'specifies source SYSTEM user'),
     ('N', 'targetnode'             , ':', socket.gethostname()  , 'specifies target servernode '),
     ('D', 'targetdbname'           , ':', 'targetdb'            , 'specifies target serverdb to create'),
     ('A', 'targetdbauser'          , ':', 'DBA,DBA'             , 'specifies target SYSDBA user'),
     ('S', 'targetdbmuser'          , ':', 'DBM,DBM'             , 'specifies target SYSTEM user'),
     #Transformation Attributes
     ('t', 'transformationtype'     , ':', 'transport'           , 'type         of transformation '),             # IMPORT , EXPORT, TRANSPORT
     ('p', 'transformationpackage'  , ':', 'db'                  , 'package      of transformation '),             # DB     , USER,   TABLE
     ('m', 'streammedium'           , ':', 'file'                , 'streammedium of transformation '),             # PIPE   , FILE,   TAPE
     ('f', 'streamformat'           , ':', 'records'             , 'streamformat of transformation '),             # RECORDS, PAGES,  COMPRESSED, FORMATTED, FORMATTED, DDL, XML
     ('F', 'streamformat_target'    , ':', 'data'                , 'streamformat of transformation '),             # DATA   , FAST
     ('e', 'streamcharencoding'     , ':', 'ascii'               , 'streamcoding of transformation '),             # ASCII  , UCS2,   UTF8
     ('b', 'streamblob'             , ':', 'single'              , 'blobmask     of transformation '),             # SINGLE,  MULTIPLE
     ('l', 'restart'                , ':', 'start'               , 'restart      transformation '),                # START,   RESTART
     #Resource Stream Connector
     ('x', 'streamconnector'        , ':', 'none'                , 'streamconnector of medium '),                  # PIPE2TAPE, PIPE2FILE
     ('X', 'migration'              , ':', 'sapdb'               , 'specifies source database system'),            # SAPDB, ADABAS

     #dummy options
     ('u', 'dummy'                  , ':', ''                    , 'specifies usecase'),                           # dummy
     ('v', 'loaderinstroot'         , ':', ''                    , 'specifies loader version'),                    # depreceated
     ('i', 'sourceinit'             , ':', ''                    , 'init      sapdb  source'),                     # dummy
     ('j', 'sourceindepdatapath'    , ':', ''                    , 'specifies sapdb  source independent path'),    # depreceated
     ('r', 'sourceinstroot'         , ':', ''                    , 'specifies sapdb  source instroot    path'),    # depreceated
     ('c', 'sourcecharencoding'     , ':', ''                    , 'specifies sapdb  source char encoding'),       # depreceated
     ('I', 'targetinit'             , ':', ''                    , 'init      sapdb  target'),                     # dummy
     ('J', 'targetindepdatapath'    , ':', ''                    , 'specifies sapdb  target independent path'),    # depreceated
     ('R', 'targetinstroot'         , ':', ''                    , 'specifies sapdb  target instroot    path'),    # depreceated
     ('C', 'targetcharencoding'     , ':', ''                    , 'specifies sapdb  target char encoding'),       # depreceated
     ('v', 'validation'             , ':', ''                    , 'specifies validation scope'),                  # dummy
     ('V', 'validationmode'         , ':', ''                    , 'specifies validation type' ),                  # dummy
]

def  check_options(options):
     rc = 0

     rc = check_configuration (options)

     options.loaderinstroot  = os.environ['INSTROOT']                        # obsolet

     #Transformation Attributes
     print 'transformationtype    -t %s  '  % options.transformationtype
     print 'transformationpackage -p %s  '  % options.transformationpackage
     print 'streammedium          -m %s  '  % options.streammedium
     print 'streamformat          -f %s  '  % options.streamformat
     print 'streamformat_target   -F %s  '  % options.streamformat_target
     print 'streamcharencoding    -e %s  '  % options.streamcharencoding
     print 'streamblob            -b %s  '  % options.streamblob
     print 'restart               -l %s  '  % options.restart

     return rc

def main (options, args):

    if  check_options(options) == 0:

        try:
            if string.upper(options.demo) == 'TRUE':
               input('\nInit      Transformation Package')
            if string.upper(options.demo) == 'FALSE':
               print '\nInit      Transformation Package'
        except:
            pass

        # model 'transformationpackage'
        transformation_package = TransformationPackage( options )
        transformation         = StreamTransformation ( options )
        transformation_package.transformation = transformation
        transformation.package                = transformation_package
        transformation.sourceloader.package   = transformation_package
        transformation.targetloader.package   = transformation_package

        try:
            if string.upper(options.demo) == 'TRUE':
               input('\nLoad      Transformation Package')
            if string.upper(options.demo) == 'FALSE':
               print '\nLoad      Transformation Package'
        except:
            pass

        # model loading
        if string.upper(options.demo) == 'TRUE':
           from xml.dom            import ext
           from xml.dom.ext.reader import Sax
           packagereader = Sax.Reader()
           transformation_package.setReader( packagereader )
           transformation_package.load     ( transformation_package.transformationpackage)
           transformation_package.execute  ( transformation_package.packagedom)
           transformation_package.unload   ()

        try:
            if string.upper(options.demo) == 'TRUE':
               input('\nExecute   Transformation Package')
            if string.upper(options.demo) == 'FALSE':
               print '\nExecute   Transformation Package'
        except:
            pass

        # model execution
        packageparser = drv_xmlproc.SAX_XPParser()
        packageparser.setDocumentHandler( transformation_package)
        packageparser.parse             ( transformation_package.transformationpackage)

        mutex.acquire()     #acquire

        try:
            if string.upper(options.demo) == 'TRUE':
               input('Validate  Transformation Package')
            if string.upper(options.demo) == 'FALSE':
               print 'Validate  Transformation Package'
               i = 1/0
        except:
            pass
        time.sleep(1)
        transformation.cmd_validate_package    ()
        if mutex.locked() == 1:
           mutex.release()  #release

        try:
            if string.upper(options.demo) == 'TRUE':
               input('Purify    Transformation Package')
            if string.upper(options.demo) == 'FALSE':
               print 'Purify    Transformation Package'
               i = 1/0
        except:
            pass
        time.sleep(1)

        transformation.cmd_purify_package   ()

        try:
            transformation.sourceloader.loadersession.release()
        except loader.LoaderError, err:
            pass

        try:
            transformation.targetloader.loadersession.release()
        except loader.LoaderError, err:
            pass

class TransformationPackage (saxlib.DocumentHandler ):

        # transformationpackage model2xml
        def __init__(self, options = None ):

            self.options               = options
            self.packageloader         = None
            self.packagereader         = None
            self.packagedom            = None
            self.sourceconfiguration   = os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.options.sourcedbname, 'LOADER' , '%s.configuration' % self.options.sourcedbname)
            self.targetconfiguration   = os.path.join(self.options.targetindepdatapath,  'wrk', '%s' % self.options.targetdbname, 'LOADER' , '%s.configuration' % self.options.targetdbname)
            self.transformationpackage = None
            self.transformation        = None
            self.transformation_step   = []
            self.sapdbnode             = None
            self.sapdb                 = None
            self.user                  = None
            self.pwd                   = None
            self.usercount             = 0
            self.table                 = None
            self.tablecount            = 0
            self.index                 = None
            self.column                = ''
            self.columncount           = 0
            self.keycolumn             = None
            self.options               = options
            self.statement             = ''
            self.statement_long        = ''
            self.longcolumns           = []

            if( self.options.transformationpackage == 'DB'     or
                self.options.transformationpackage == 'USER'   or
                self.options.transformationpackage == 'TABLE'     ):

                try:
                  if options.demo == 'TRUE':
                     input('\nStart     PackageLoader')
                  if options.demo == 'FALSE':
                     print '\nStart     PackageLoader'
                     i = 1/0
                except:
                  pass

                self.packageloader =  StreamLoader(  self.options, self.options.loaderinstroot, self.options.sourcenode  , self.options.sourcedbname      , regsub.split(self.options.sourcedbauser,',')[0] , regsub.split(self.options.sourcedbauser,',')[1], self.options.sourcecharencoding,
                                                     'EXTRACT'   , self.options.streammedium  , self.options.streamformat, self.options.streamcharencoding, self.options.streamblob, self.options.restart )

                try:
                    if string.upper(options.demo) == 'TRUE':
                       input('Extract   Transformation Package')
                    if string.upper(options.demo) == 'FALSE':
                       print 'Extract   Transformation Package'
                       i = 1/0
                except:
                    pass

                packagedba     = 'USE USER %s %s SERVERDB %s ON %s ' % ( regsub.split(self.options.sourcedbauser,',')[0], regsub.split(self.options.sourcedbauser,',')[1], self.options.sourcedbname, self.options.sourcenode )
                try :
                    self.packageloader.cmd  = packagedba
      	            self.packageloader.user = regsub.split(self.options.sourcedbauser,',')[0]
    	            self.packageloader.loadersession.cmd ("%s" % self.packageloader.cmd )
    	        except loader.LoaderError, err:
 	            print 'command failed: ', err

                package = "\nEXTRACT CATALOG ALL OUTSTREAM FILE '%s' XML \n" % os.path.join(options.sourceindepdatapath, 'wrk', '%s' % options.sourcedbname, 'LOADER', '%s.xml' % options.sourcedbname )
                try :
                    self.packageloader.cmd =  package
 	            self.packageloader.loadersession.cmd ("%s" % self.packageloader.cmd )
    	        except loader.LoaderError, err:
 	            print 'command failed: ', err

    	        try:
    	           self.packageloader.loadersession.release()
    	        except loader.LoaderError, err:
    	            print 'command failed:' , err

                self.transformationpackage  = os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % options.sourcedbname, 'LOADER', '%s.xml' % options.sourcedbname )

            elif self.options.transformationpackage == 'WEBDAV' :
                 self.transformationpackage = os.path.join(options.sourceinstroot, 'env' , '%s.xml' % self.options.transformationpackage )
                 self.options.transformationpackage  = 'TABLE'

        # model execution using dom
        def setReader(self,package_reader = None):
            self.packagereader = package_reader

        def load   (self,transformation_package = None ):
            self.packagedom    = self.packagereader.fromUri(transformation_package)
        def execute(self,package_dom = None ):
            for user in package_dom.getElementsByTagName ('sapdbm:DeployedSchema'):
                print 'User      %s' %user.getAttribute  ('oim:name')
            for table in package_dom.getElementsByTagName('sapdbm:DeployedTable' ):
                print 'Table     %s' %table.getAttribute ('oim:name')
        def unload (self):
            self.packagereader.releaseNode(self.packagedom)

        # model execution using sax
        def startElement (self, name, attrs):
            #Datasource
            if name == 'dbm:DataSource':
               pass
	    #Connection
	    if name == 'sapdbm:Connection':
	       connectString =  map(string.strip, string.split(attrs.get('dbm:ConnectString'  , None),";"))
               pos           = 1
	       for i in connectString:
         	    if pos == 1 :
         	       pass
		    if pos == 2 :
              	       self.sapdbnode  =  ''
            	    if pos == 3 :
                       self.sapdb      =  self.options.sourcedbname
            	    if pos == 4 :
                       self.user       =  regsub.split(self.options.sourcedbauser,',')[0]
            	    if pos == 5 :
                       self.pwd        = 'initial'
  	            pos = pos + 1
	    #Catalog
	    if name == 'dbm:DeployedCatalog'   :
	       sourcedba     = 'USE USER %s %s SERVERDB %s ON %s ' % ( regsub.split(self.options.sourcedbauser,',')[0], regsub.split(self.options.sourcedbauser,',')[1], self.options.sourcedbname, self.options.sourcenode )
               try :
                   self.transformation.sourceloader.cmd  = sourcedba
 	           self.transformation.sourceloader.cmd_execute ()
 	           self.transformation.sourceloader.user = regsub.split(self.options.sourcedbauser,',')[0]
               except loader.LoaderError, err:
    	           print 'command failed:' , err

    	       targetdba     = 'USE USER %s %s SERVERDB %s ON %s ' % ( regsub.split(self.options.targetdbauser,',')[0],regsub.split(self.options.targetdbauser,',')[1], self.options.targetdbname, self.options.targetnode )
               try :
                   self.transformation.targetloader.cmd  = targetdba
 	           self.transformation.targetloader.cmd_execute ()
 	           self.transformation.targetloader.user = regsub.split(self.options.targetdbauser,',')[0]
               except loader.LoaderError, err:
    	           print 'command failed:' , err

            #Schema
            if name == 'sapdbm:DeployedSchema' :
               self.user = attrs.get('oim:name',None )
               if self.user == regsub.split(self.options.sourcedbauser,',')[0]:
                  self.pwd   = regsub.split(self.options.sourcedbauser,',')[1]
               else:
                  self.pwd   = 'initial'

               self.transformation.sourceloader.cmd = ''
               self.transformation.targetloader.cmd = ''

               if(self.options.migration             == 'ADABAS' or
                  self.options.transformationpackage == 'USER'   or
                  self.options.transformationpackage == 'TABLE'):

                  #Source
                  cmd = 'USE USER %s %s SERVERDB %s ON %s ' % ( self.user, self.pwd , self.options.sourcedbname, self.options.sourcenode )
                  try :
                      self.transformation.sourceloader.cmd  = cmd
 	              self.transformation.sourceloader.cmd_execute ()
 	              self.transformation.sourceloader.user = self.user
                  except loader.LoaderError, err:
    	              print 'command failed:' , err

                  #Target
    	          cmd = 'USE USER %s %s SERVERDB %s ON %s ' % ( self.user, self.pwd , self.options.targetdbname, self.options.targetnode )
                  try :
                      self.transformation.targetloader.cmd  = cmd
 	              self.transformation.targetloader.cmd_execute ()
 	              self.transformation.targetloader.user = self.user
                  except loader.LoaderError, err:
    	              print 'command failed:' , err
    	          time.sleep(1)
 	        #Table
            if name == 'sapdbm:DeployedTable'  :
               self.table      = attrs.get('oim:name', None)
               self.tablecount = self.tablecount +1
               self.transformation.sourceloader.cmd = ''
               self.transformation.targetloader.cmd = ''
               self.statement                       = ''
               self.statement_long                  = ''
               self.statement_column                = ''
               self.longcolumns                     = []
            if name == 'dbm:TableConstraint':
               pass
            if name == 'dbm:Index':
               self.index = attrs.get('dbm:name', None)
               pass
            if name == 'dbm:ColumnSetColumns':
               pass
            if name == 'dbm:KeyColumns' :
               self.keycolumn = 'true'
               pass
            if name == 'dbm:DeployedColumn'  :
               self.column = attrs.get('dbm:name', None)
            if name == 'sapdbm:LONG'      and self.options.streamformat   <> 'RECORDS' :
               if self.column <> ''       and self.options.streammedium   == 'FILE'    :
                  self.statement_long = self.statement_long + " LONGSTREAM FILE %s '%s' %s \n" % ( self.column, os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.sapdb , 'LOADER', '%s_%s_%s_%s.long%s' % (self.sapdb,self.user,self.table,self.column,self.transformation.sourceloader.blobmask)),self.options.streamcharencoding)
               if self.column <> ''       and ( self.options.streammedium == 'PIPE' or  self.options.streammedium  == 'PIPE2FILE' ) :
                  self.statement_long = self.statement_long + " LONGSTREAM FILE %s '%s' %s \n" % ( self.column, '\\\\.\\PIPE\\%s_%s_%s_%s.long' % (self.sapdb,self.user,self.table,self.column),self.options.streamcharencoding)
                  self.longcolumns.append(self.column)
            if name == 'sapdbm:LONG_BYTE' and self.options.streamformat   <> 'RECORDS' :
               if self.column <> ''       and self.options.streammedium   == 'FILE'    :
                  self.statement_long = self.statement_long + " LONGSTREAM FILE %s '%s' \n" % ( self.column, os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.sapdb , 'LOADER', '%s_%s_%s_%s.long%s' % (self.sapdb,self.user,self.table,self.column,self.transformation.sourceloader.blobmask)))
               if self.column <> ''       and ( self.options.streammedium == 'PIPE' or  self.options.streammedium  == 'PIPE2FILE' ) :
                  self.statement_long = self.statement_long + " LONGSTREAM FILE %s '%s' \n" % ( self.column, '\\\\.\\PIPE\\%s_%s_%s_%s.long' % (self.sapdb,self.user,self.table,self.column))
                  self.longcolumns.append(self.column)
               self.column = ''
            if name == 'dbm:UniqueKey':
               pass
            if name == 'dbm:UniqueKeyRole':
               pass
            if name == 'dbm:ForeignKey':
               pass
            if name == 'dbm:ForeignKeyRole':
               pass

        def endElement(self, name ):
            #Catalog
            if name == 'dbm:DeployedCatalog' :
               #Part
               if self.options.transformationpackage     == 'DB'   :
                  mutex.acquire()                                           #acquire
                  input  = None
                  #Format
                  if(self.options.streamformat        == 'XML'     or
                     self.options.streamformat        == 'DDL'     or
                     self.options.streamformat        == 'PAGES'   or
                     self.options.streamformat        == 'RECORDS'  ):

                    #Type
                     if self.options.transformationtype    == 'TRANSPORT':
                        while input not in ["y","n"]:
                              if self.options.demo == 'TRUE':
                                 input = raw_input('Transport DB %s ? [y]/[n] : ' % ( self.sapdb ) )
                              if self.options.demo == 'FALSE':
                                 print             'Transport DB %s' % ( self.sapdb )
                                 input  = "y"
                              if input == "y":
                                 self.transformation.cmd_transport_catalog_package()
                              if input == "n":
                                 if mutex.locked() == 1:
                                    mutex.release()
                     if(self.options.transformationtype    == 'EXPORT'  and
                        self.options.streammedium          == 'FILE'     ):
                        while input not in ["y","n"]:
                              if self.options.demo == 'TRUE':
                                 input = raw_input('Export    DB %s ? [y]/[n] : ' % ( self.sapdb ))
                              if self.options.demo == 'FALSE':
                                 print             'Export    DB %s' % ( self.sapdb )
                                 input  = "y"
                              if input == "y":
                                 self.transformation.cmd_export_catalog_package()
                              if input == "n":
                                 if mutex.locked() == 1:
                                    mutex.release()
                     if(self.options.transformationtype     == 'IMPORT' and
                        self.options.streammedium           == 'FILE'    ):
                        while input not in ["y","n"]:
                              if self.options.demo == 'TRUE':
                                 input = raw_input('Import    DB %s ? [y]/[n] : ' % (self.sapdb ))
                              if self.options.demo == 'FALSE':
                                 print             'Import    DB %s' % (self.sapdb )
                                 input  = "y"
                              if input == "y":
                                 self.transformation.cmd_import_catalog_package()
                              if input == "n":
                                 if mutex.locked() == 1:
                                    mutex.release()                        #release
               self.user = ''


            #Schema
            if name == 'sapdbm:DeployedSchema':
               #Part
               if(self.options.transformationpackage  == 'USER'    and
                  self.user                           <> 'DBA'       ) :
                  mutex.acquire()                                           #acquire
                  input  = None

                  #Format
                  if(self.options.streamformat        == 'XML'     or
                     self.options.streamformat        == 'DDL'     or
                     self.options.streamformat        == 'PAGES'   or
                     self.options.streamformat        == 'RECORDS'  ):

                     #Type
                     if self.options.transformationtype    == 'TRANSPORT':
                        while input not in ["y","n"]:
                              if self.options.demo == 'TRUE':
                                 input = raw_input('Transport User %s ? [y]/[n] : ' % ( self.user ) )
                              if self.options.demo == 'FALSE':
                                 print             'Transport User %s' % ( self.user )
                                 input  = "y"
                              if input == "y":
                                 self.transformation.cmd_transport_schema_package()
                              if input == "n":
                                 if mutex.locked() == 1:
                                    mutex.release()
                     if(self.options.transformationtype    == 'EXPORT'  and
                        self.options.streammedium          == 'FILE'     ):
                        while input not in ["y","n"]:
                              if self.options.demo == 'TRUE':
                                 input = raw_input('Export    User %s ? [y]/[n] :  ' % ( self.user ))
                              if self.options.demo == 'FALSE':
                                 print             'Export    User %s' % ( self.user )
                                 input  = "y"
                              if input == "y":
                                 self.transformation.cmd_export_schema_package()
                              if input == "n":
                                 if mutex.locked() == 1:
                                    mutex.release()
                     if(self.options.transformationtype     == 'IMPORT' and
                        self.options.streammedium           == 'FILE'    ):
                        while input not in ["y","n"]:
                              if self.options.demo == 'TRUE':
                                 input = raw_input('Import    User %s ? [y]/[n] :  ' % (self.user ))
                              if self.options.demo == 'FALSE':
                                 print             'Import    User %s' % (self.user )
                                 input  = "y"
                              if input == "y":
                                 self.transformation.cmd_import_schema_package()
                              if input == "n":
                                 if mutex.locked() == 1:
                                    mutex.release()                        #release
               self.user = ''
            #Table
            if name == 'sapdbm:DeployedTable' :
               #Part
               if self.options.transformationpackage   == 'TABLE' :
                  mutex.acquire()                                          #acquire
                  input  = None
                  #Format
 	          if(self.options.streamformat         == 'XML'       or
                     self.options.streamformat         == 'DDL'       or
 	             self.options.streamformat         == 'PAGES'     or
 	             self.options.streamformat         == 'RECORDS'   or
 	             self.options.streamformat[0:9]    == 'FORMATTED' or
                     self.options.streamformat         == 'COMPRESSED' ):

                     #Type
                     if self.options.transformationtype    == 'TRANSPORT':
                        while input not in ["y","n"]:
                              if self.options.demo == 'TRUE':
                                 input = raw_input('Transport Table %s ? [y]/[n] : ' % ( self.table ) )
                              if self.options.demo == 'FALSE':
                                 print             'Transport Table %s' % ( self.table )
                                 input  = "y"
                              if input == "y":
                                 self.transformation.cmd_transport_table_package()
                              if input == "n":
                                 if mutex.locked() == 1:
                                    mutex.release()
                     if(self.options.transformationtype    == 'EXPORT'  and
                        self.options.streammedium          == 'FILE'     ):
                        while input not in ["y","n"]:
                              if self.options.demo == 'TRUE':
                                 input = raw_input('Export    Table %s ? [y]/[n] : ' % ( self.table ))
                              if self.options.demo == 'FALSE':
                                 print             'Export    Table %s' % ( self.table )
                                 input  = "y"
                              if input == "y":
                                 self.transformation.cmd_export_table_package()
                              if input == "n":
                                 if mutex.locked() == 1:
                                    mutex.release()
                     if(self.options.transformationtype     == 'IMPORT' and
                        self.options.streammedium           == 'FILE'    ):
                        while input not in ["y","n"]:
                              if self.options.demo == 'TRUE':
                                 input = raw_input('Import    Table %s ? [y]/[n] : ' % (self.table ))
                              if self.options.demo == 'FALSE':
                                 print             'Import    Table %s' % (self.table )
                                 input  = "y"
                              if input == "y":
                                 self.transformation.cmd_import_table_package()
                              if input == "n":
                                 if mutex.locked() == 1:
                                    mutex.release()                       #release

               #Part
               if self.options.migration               == 'ADABAS'  :
                  #Format migration
                  if(self.options.streamformat         == 'RECORDS'    or
                     self.options.streamformat[0:9]    == 'FORMATTED'  or
                     self.options.streamformat         == 'COMPRESSED'   ):
                     #Medium
                     if self.options.streammedium    == 'FILE'  :
 	                self.statement = 'EXTRACT DATA FOR LOAD %s NODDL TABLE '             % (self.options.streamformat_target)
 	                self.statement = self.statement + " \"%s\".\"%s\"                \n" % (self.user, self.table)
 	                self.statement = self.statement + " OUTSTREAM FILE '%s' APPEND   \n" % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.sapdb , 'LOADER', '%s.dtl'        % (self.sapdb)))
 	                self.statement = self.statement + " OUTSTREAM FILE '%s' %s       \n" % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.sapdb , 'LOADER', '%s_%s_%s.data' % (self.sapdb,self.user,self.table )),self.options.streamformat)
 	                self.statement = self.statement + self.statement_long
 	                #export
                        self.transformation.sourceloader.cmd = self.statement
                        self.transformation.sourceloader.cmd_execute()

                     self.statement = 'EXTRACT DATA \n'
                     self.statement = self.statement + self.statement_column
                     self.statement = self.statement + " FROM \"%s\".\"%s\" ;         \n"   % (self.user, self.table)
                     self.statement = self.statement + " OUTFILE '%s' COMPRESS        \n"   % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.sapdb , 'LOADER', '%s_%s_%s.data' % (self.sapdb,self.user,self.table )))
                     self.statement = self.statement + self.statement_long
                     self.statement = string.replace(self.statement, ',\n FROM', '\n FROM' )
                     adabasdata_bat =                                            open('%s'  %  os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.sapdb , 'LOADER' , 'adabas_data.bat' ),'a+')
                     adabasdata_bat.write("//\n")
                     adabasdata_bat.write(self.statement)
                     adabasdata_bat.close
               pass
            if name == 'dbm:KeyColumns' :
               self.keycolumn = None
               pass
            if name == 'dbm:ColumnSetColumns' :
               pass
            if name == 'dbm:DeployedColumn'  :
               if self.keycolumn == None :
                  self.statement_column  = self.statement_column + "\"%s\",\n" % self.column
               pass
            if name == 'sapdbm:LONG' :
               pass
            if name == 'sapdbm:LONG_BYTE':
               pass
            if name == 'dbm:Index':
               if(self.options.streamformat_target == 'FAST' and
                  self.statement_long == '' ):
                  self.transformation.targetloader.cmd     =  'DROP INDEX \"%s\" ON \"%s\".\"%s\" \n' % (self.index, self.user, self.table)
                  self.transformation.targetloader.cmd_execute()
                  self.transformation.targetloader.cmd     =  ''
               pass
            if name == 'dbm:IndexIndexColumns':
               pass
            if name == 'dbm:TableSynonym' :
               pass

class StreamTransformation :

      def __init__( self, options = None ) :
          self.options                = options
          self.package                = None
          self.sourcesession          = None
          self.sourceloader           = None
          self.targetloader           = None

          # source session
          self.sourcesession = Session(options)

          # stream loader source
          try:
              if options.demo == 'TRUE':
                 input('\nStart     SourceLoader')
              if options.demo == 'FALSE':
                 print '\nStart     SourceLoader'
                 i = 1/0
          except:
              pass

          self.sourceloader =  StreamLoader(  self.options, options.loaderinstroot   , self.options.sourcenode  , self.options.sourcedbname      , regsub.split(self.options.sourcedbauser,',')[0], regsub.split(self.options.sourcedbauser,',')[1], self.options.sourcecharencoding,
                                              'EXTRACT'   , self.options.streammedium, self.options.streamformat, self.options.streamcharencoding, self.options.streamblob, self.options.restart )
          time.sleep(1)

          # stream loader target
          try:
              if options.demo == 'TRUE':
                 input('\nStart     TargetLoader')
              if options.demo == 'FALSE':
                 print '\nStart     TargetLoader'
                 i = 1/0
          except:
              pass

          self.targetloader =  StreamLoader(  self.options, options.loaderinstroot  , self.options.targetnode  , self.options.targetdbname      , regsub.split(self.options.targetdbauser,',')[0] ,regsub.split(self.options.targetdbauser,',')[1], self.options.targetcharencoding,
                                              'LOAD'     , self.options.streammedium, self.options.streamformat, self.options.streamcharencoding, self.options.streamblob, self.options.restart )
          time.sleep(1)

      def cmd_release_execute_package(self):
          # trigger package and release streamloader
          release        = 'FALSE'
          while release == 'TRUE':
                try:
                    cmd  = "SELECT COUNT(*) FROM SYSDBA.TRANSFORMATIONMODEL WHERE RESTART = FALSE "
                    self.sourcesession.cmd = cmd
                    self.sourcesession.cmd_execute()
                    if self.sourcesession.result[0] == 1 :
                       print '\nStop   Package Execution'
                       self.sourceloader.loadersession.release()  # python->cancelbyte
                       self.targetloader.loadersession.release()  # python->cancelbyte
                       release = 'TRUE'
                except sapdb.SQLError,  err :
                    print 'sql command failed:', err
                except loader.LoaderError, err :
                    print 'loader command failed:', err
                except loader.CommunicationError, err :
                    print 'loader communication failed:', err

      def cmd_transport_catalog_package(self):
          input = None
          # stream connector and stream filter
          self.streamconnector =  StreamConnector(self.sourceloader, self.targetloader )
          self.streamfilter    =  StreamFilter   (self.sourceloader, self.targetloader )

          #stream package
          if self.options.restart == 'RESTART':
             try:
                 if options.demo == 'TRUE':
                    input('Start init Transformationpackages')
                 if options.demo == 'FALSE':
                    print 'Start init Transformationpackages'
                    i = 1/0
             except:
                 restartRecord = self.targetloader.cmd_restart_targetdb_package()
                 self.sourceloader.cmd_restart_sourcedb_package ( restartRecord )

          #stream export
          while input not in ["y"]:
                if self.options.demo == 'TRUE':
                   input = raw_input('Export    DB %s ? [y]     : ' % ( self.package.sapdb ))
                if self.options.demo == 'FALSE':
                   input   = "y"
                if input  == "y":
                   #stream loader
                   thread.start_new_thread( self.sourceloader.cmd_use_serverdb_user      ,())
                   time.sleep(1)
                   thread.start_new_thread( self.sourceloader.cmd_export_catalog_package ,()) #export
                   time.sleep(1)

                   #stream connector persistent
                   if self.options.streammedium == 'PIPE2TAPE' :
                      thread.start_new_thread( self.streamconnector.cmd_ddlpipe2ddltape  ,())
                      time.sleep(1)
                      thread.start_new_thread( self.streamconnector.cmd_datpipe2dattape  ,())
                      time.sleep(1)
                   if self.options.streammedium == 'PIPE2FILE' :
                      thread.start_new_thread( self.streamconnector.cmd_ddlpipe2ddlfile  ,())
                      time.sleep(1)
                      thread.start_new_thread( self.streamconnector.cmd_datpipe2datfile  ,())
                      time.sleep(1)

                   #stream package
                   if self.options.streammedium == 'FILE' and self.options.streamformat <> 'XML':
                      while self.sourceloader.packageunload == 'FALSE' :
                            thread.start_new_thread( self.sourceloader.cmd_unload_package,()) #release
                            time.sleep(1)
                      mutex.acquire()                                                         #acquire
                   if self.options.streammedium == 'TAPE' :
                      while self.sourceloader.packageunload == 'FALSE' :
                            thread.start_new_thread( self.sourceloader.cmd_unload_package,()) #release
                            time.sleep(1)
                      mutex.acquire()                                                         #acquire
                      self.targetloader.ddlpipe_ebid = self.sourceloader.ddlpipe_ebid
                      self.targetloader.datpipe_ebid = self.sourceloader.datpipe_ebid
                      self.targetloader.mappipe_ebid = self.sourceloader.mappipe_ebid

          input = None

          #stream import
          while input not in ["y"]:
                if self.options.demo == 'TRUE':
                   input = raw_input('Import    DB %s ? [y]     : ' % ( self.package.sapdb ))
                if self.options.demo == 'FALSE':
                   input   = "y"
                if input  == "y":
                   #stream loader
                   thread.start_new_thread( self.targetloader.cmd_use_serverdb_user      ,())
                   time.sleep(1)
                   thread.start_new_thread( self.targetloader.cmd_import_catalog_package ,())  #import
                   time.sleep(1)

                   #stream connector transient
                   if ( string.upper(self.options.streammedium)     == 'PIPE'      or
                        string.upper(self.options.streammedium)     == 'PIPE2TAPE' ):
                        # stream filter
                        ddlfilter = 0
                        if ddlfilter == 0:
                           while  self.streamfilter.ddlerror  == 1 :
                                  thread.start_new_thread( self.streamfilter.cmd_ddlsource2ddltarget    ,())
                                  time.sleep(1)
                        datfilter = 1
                        if datfilter == 0:
                           while  self.streamfilter.daterror  == 1 :
                                  thread.start_new_thread( self.streamfilter.cmd_datsource2dattarget_bin,())
                                  time.sleep(1)
                   #stream connector persistent
                   if ( string.upper(self.options.streammedium)     == 'PIPE2TAPE' or
                        string.upper(self.options.streammedium)     == 'PIPE2FILE'    ):
                        ddlfilter = 1
                        datfilter = 1
                        if  string.upper(self.options.streammedium) == 'PIPE2TAPE'  :
                            thread.start_new_thread( self.streamconnector.cmd_ddltape2ddlpipe,())
                            time.sleep(1)
                            thread.start_new_thread( self.streamconnector.cmd_dattape2datpipe,())
                            time.sleep(1)
                            ddlfilter = 0
                        if  string.upper(self.options.streammedium) == 'PIPE2FILE'  :
                            thread.start_new_thread( self.streamconnector.cmd_ddlfile2ddlpipe,())
                            time.sleep(1)
                            thread.start_new_thread( self.streamconnector.cmd_datfile2datpipe,())
                            time.sleep(1)
                            ddlfilter = 0
                        # stream filter
                        if ddlfilter == 0:
                           while  self.streamfilter.ddlerror  == 1 :
                                  thread.start_new_thread( self.streamfilter.cmd_ddlsource2ddltarget    ,())
                                  time.sleep(10)
                        if datfilter == 0:
                           while  self.streamfilter.daterror  == 1 :
                                  thread.start_new_thread( self.streamfilter.cmd_datsource2dattarget_bin,())
                                  time.sleep(10)
                   #stream package
                   if self.options.streammedium == 'PIPE' :
                      while self.sourceloader.packageunload == 'FALSE' :
                            thread.start_new_thread( self.sourceloader.cmd_unload_package,()) #release
                            time.sleep(1)
                      while self.targetloader.packageunload == 'FALSE' :
                            thread.start_new_thread( self.targetloader.cmd_unload_package,()) #release
                            time.sleep(1)
                   if self.options.streammedium == 'FILE' :
                      while self.targetloader.packageunload == 'FALSE' :
                            thread.start_new_thread( self.targetloader.cmd_unload_package,()) #release
                            time.sleep(1)
                   if self.options.streammedium == 'TAPE' :
                      while self.targetloader.packageunload == 'FALSE' :
                            thread.start_new_thread( self.targetloader.cmd_unload_package,()) #release
                            time.sleep(1)

          self.sourceloader.packageload   = 'FALSE'
          self.sourceloader.packageunload = 'FALSE'
          self.targetloader.packageload   = 'FALSE'
          self.targetloader.packageunload = 'FALSE'

      def cmd_export_catalog_package(self):
          #stream packages
          self.sourceloader.cmd_export_catalog_package()
          if  mutex.locked()  == 1 :
              mutex.release()           #release

      def cmd_import_catalog_package(self):
          #stream packages
          self.targetloader.cmd_import_catalog_package()
          if  mutex.locked()  == 1 :
              mutex.release()           #release

      def cmd_transport_schema_package(self):
          input = None
          # stream connector and stream filter
          self.streamconnector =  StreamConnector(self.sourceloader, self.targetloader )
          self.streamfilter    =  StreamFilter   (self.sourceloader, self.targetloader )

          #stream packages
          if self.options.restart == 'RESTART':
             try:
                 if string.upper(options.demo)  == 'TRUE':
                    input('Start init Transformationpackages')
             except:
                 restartRecord = self.targetloader.cmd_restart_targetdb_package()
                 self.sourceloader.cmd_restart_sourcedb_package( restartRecord )

          #stream export
          while input not in ["y"]:
                if self.options.demo == 'TRUE':
                   input = raw_input('Export    Schema %s ? [y] : ' % ( self.package.user ))
                if self.options.demo == 'FALSE':
                   input   = "y"
                if input  == "y":
                   #stream loader
                   thread.start_new_thread( self.sourceloader.cmd_export_schema_package  ,()) #export

                   if self.options.streammedium == 'FILE' :
                      while  self.sourceloader.packageload   == 'FALSE' :
                             thread.start_new_thread( self.sourceloader.cmd_load_package    ,())
                             time.sleep(1)
                      while  self.sourceloader.packageunload == 'FALSE' :
                             thread.start_new_thread( self.sourceloader.cmd_unload_package  ,()) #release
                             time.sleep(1)
                      if self.options.streamformat <> 'XML':
                         mutex.acquire()

                   if self.options.streammedium == 'TAPE' :
                      mutex.acquire()
                      self.targetloader.ddlpipe_ebid = self.sourceloader.ddlpipe_ebid
                      self.targetloader.datpipe_ebid = self.sourceloader.datpipe_ebid
                      self.targetloader.mappipe_ebid = self.sourceloader.mappipe_ebid

                   # stream connector
                   if self.options.streammedium == 'PIPE2TAPE' :
                      thread.start_new_thread( self.streamconnector.cmd_ddlpipe2ddltape,())
                      time.sleep(1)
                      thread.start_new_thread( self.streamconnector.cmd_datpipe2dattape,())
                      time.sleep(1)
                   if self.options.streammedium == 'PIPE2FILE' :
                      thread.start_new_thread( self.streamconnector.cmd_ddlpipe2ddlfile,())
                      time.sleep(1)
                      thread.start_new_thread( self.streamconnector.cmd_datpipe2datfile,())
                      time.sleep(1)

          input = None

          # stream import
          while input not in ["y"]:
                if self.options.demo == 'TRUE':
                   input = raw_input('Import    Schema %s ? [y] : ' % ( self.package.user ))
                if self.options.demo == 'FALSE':
                   input   = "y"
                if input  == "y":
                   #stream loader
                   thread.start_new_thread( self.targetloader.cmd_import_schema_package ,()) #import
                   time.sleep(1)
                   while  self.targetloader.packageload   == 'FALSE' :
                          thread.start_new_thread( self.targetloader.cmd_load_package   ,())
                          time.sleep(1)
                   while  self.targetloader.packageunload == 'FALSE' :
                          thread.start_new_thread( self.targetloader.cmd_unload_package ,()) #release
                          time.sleep(1)

          self.sourceloader.packageload   = 'FALSE'
          self.sourceloader.packageunload = 'FALSE'
          self.targetloader.packageload   = 'FALSE'
          self.targetloader.packageunload = 'FALSE'

      def cmd_export_schema_package(self):
          #stream package
          self.sourceloader.cmd_export_schema_package()
          if  mutex.locked()  == 1 :
              mutex.release()           #release

      def cmd_import_schema_package(self):
          #stream package
          self.targetloader.cmd_import_schema_package()
          if  mutex.locked()  == 1 :
              mutex.release()           #release

      def cmd_transport_table_package(self):
          input = None
          #  stream filter
          self.streamfilter    =  StreamFilter   (self.sourceloader, self.targetloader )

          #Medium
          streamformat        = self.options.streamformat
          streamcharencoding  = self.options.streamcharencoding
          #Format
          streamformat_target = self.options.streamformat_target

          if self.package.statement_long <> '' :
             if((self.options.streamformat     == 'FORMATTED' or
                 self.options.streamformat     == 'COMPRESSED'  )):
                 if self.options.streammedium  == 'PIPE' :
                    self.options.streamformat   = 'RECORDS'
                 if self.options.streamformat_target == 'FAST' :
                    self.options.streamformat_target  = 'DATA'
             if self.options.streamformat      == 'RECORDS':
                if self.options.streamcharencoding   == 'UTF8':
                   self.options.streamcharencoding    = 'UCS2'

          #stream export
          while input not in ["y"]:
                if self.options.demo == 'TRUE':
                   input = raw_input('Export    Table %s ? [y] : ' % ( self.package.table ))
                if self.options.demo == 'FALSE':
                   input   = "y"
                if input  == "y":
                   #stream loader
                   thread.start_new_thread( self.sourceloader.cmd_export_table_package, ())    #export
                   time.sleep(1)

                   if  self.options.streammedium == 'FILE'  :
                       #stream package
                       while  self.sourceloader.packageunload == 'FALSE' :
                              thread.start_new_thread( self.sourceloader.cmd_unload_package,())#release
                              time.sleep(1)
                       mutex.acquire()

          #stream filter
          if ( self.options.streamformat  == 'DDL'       or
               self.options.streamformat  == 'FORMATTED' or
               self.options.streamformat  == 'COMPRESSED'   ) :
               while  self.streamfilter.dtlerror == 1 :
                      thread.start_new_thread( self.streamfilter.cmd_dtlsource2dtltarget,())
                      time.sleep(1)

          input = None

          #stream import
          while input not in ["y"]:
                if self.options.demo == 'TRUE':
                   input = raw_input('Import    Table %s ? [y] : ' % ( self.package.table ))
                if self.options.demo == 'FALSE':
                   input   = "y"
                if input  == "y":
                   #stream loader
                   thread.start_new_thread( self.targetloader.cmd_import_table_package, ())    #import

                   #stream package
                   while( self.targetloader.packageunload == 'FALSE'):
                          thread.start_new_thread( self.targetloader.cmd_unload_package,())    #release
                          time.sleep(1)

          #Medium
          if streamformat != self.options.streamformat :
             self.options.streamformat = streamformat
          if streamcharencoding != self.options.streamcharencoding :
             self.options.streamcharencoding = streamcharencoding
          #Format
          if streamformat_target != self.options.streamformat_target :
             self.options.streamformat_target = streamformat_target

          self.sourceloader.packageload   = 'FALSE'
          self.sourceloader.packageunload = 'FALSE'
          self.targetloader.packageload   = 'FALSE'
          self.targetloader.packageunload = 'FALSE'

      def cmd_export_table_package(self):
          #stream package
          self.sourceloader.cmd_export_table_package()
          if  mutex.locked()  == 1 :
              mutex.release()           #release

      def cmd_import_table_package(self):
          # stream connector and stream filter
          self.streamconnector =  StreamConnector(self.sourceloader, self.targetloader )
          self.streamfilter    =  StreamFilter   (self.sourceloader, self.targetloader )

          #stream package
          self.streamfilter.cmd_dtlsource2dtltarget()
          self.targetloader.cmd_import_table_package()
          if  mutex.locked()  == 1 :
              mutex.release()           #release

      def cmd_validate_package(self):
          sourcepackage = None
          targetpackage = None
          if  self.options.transformationpackage  == 'DB' :

              for file in os.listdir(os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.options.sourcedbname , 'LOADER')) :
                   if (string.find(file,'package') <> -1 ):
                      sourcepackage =  'LOAD DATA TABLE TRANSFORMATIONMODEL  \n'
                      sourcepackage = sourcepackage + 'OWNER                 1 DEFAULT      NULL TABLENAME            2 DEFAULT      NULL \nDATASTREAMNAME          3 DEFAULT      NULL \n'
                      sourcepackage = sourcepackage + 'DATASTREAMHEADERBLOCK 4 DEFAULT      NULL DATASTREAMHEADERPOS  5 DEFAULT      NULL \nDATASTREAMTRAILERBLOCK  6 DEFAULT      NULL DATASTREAMTRAILERPOS  7 DEFAULT      NULL  DATASTREAMDATABLOCK  8 DEFAULT      NULL DATASTREAMDATAPOS    9  DEFAULT    NULL \n'
                      sourcepackage = sourcepackage + '\"EXCLUDE\"          10 DEFAULT      NULL \n'
                      sourcepackage = sourcepackage + 'PACKAGEGUID          11 DEFAULT      NULL TASKID              12 DEFAULT      NULL \nSTEPID                 13 DEFAULT      NULL SERVERNODE           14 DEFAULT      NULL \n'
                      sourcepackage = sourcepackage + '\"SERVERDB\"         15 DEFAULT      NULL USERNAME            16 DEFAULT      NULL \n\"CODETYPE\"           17 DEFAULT      NULL CODEPAGEID           18 DEFAULT      NULL \n'
                      sourcepackage = sourcepackage + '\"STORAGEDEVICE\"    19 DEFAULT      NULL TRANSACTIONSIZE     20 DEFAULT      NULL TRANSFORMATION           21 DEFAULT      NULL \nPART               22 DEFAULT      NULL TABLETYPE            23 DEFAULT      NULL \n'
                      sourcepackage = sourcepackage + 'CATALOGSTREAMNAME    24 DEFAULT      NULL CATALOGSTREAMTYPE   25 DEFAULT      NULL \nCATALOGSTREAMEBID      26 DEFAULT      NULL CATALOGSTREAMFORMAT  27 DEFAULT      NULL DATASTREAMTYPE       28 DEFAULT      NULL  DATASTREAMEBID     29 DEFAULT     NULL \n'
                      sourcepackage = sourcepackage + 'DATASTREAMFORMAT     30 DEFAULT      NULL \"RESTART\"         31 DEFAULT      NULL \nRESTORE                32 DEFAULT      NULL PAGECOUNT            33 DEFAULT      NULL \n'
                      sourcepackage = sourcepackage + 'ROWCOUNT             34 DEFAULT      NULL STARTTIME           35 DEFAULT      NULL \nENDTIME                36 DEFAULT      NULL REQUESTTIME          37 DEFAULT      NULL \n'
                      sourcepackage = sourcepackage + 'RECEIVETIME          38 DEFAULT      NULL IOTIME              39 DEFAULT      NULL \nLOGTIME                40 DEFAULT      NULL '
                      sourcepackage = sourcepackage + " INSTREAM FILE '%s'  COMPRESSED %s \n" % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s' % (file )),self.options.streamcharencoding )

                      print 'Load      Transformation Package x\'%s\' of EXTRACT TRANSFORMATION'   %  self.sourceloader.packageguid
                      try :
 	                  self.targetloader.loadersession.cmd ("%s" %  sourcepackage  )
	              except loader.LoaderError, err:
    	                  print 'command failed:' , err

    	      for file in os.listdir(os.path.join(self.options.targetindepdatapath,  'wrk', '%s' % self.options.targetdbname , 'LOADER')):
    	          if (string.find(file,'package')  <> -1 ):
    	              if self.targetloader.catalog == 'UNICODE' :
                         self.targetloader.package  = 'UCS2'
                      if self.targetloader.catalog == 'ASCII' :
                         self.targetloader.package  = 'ASCII'
    	              targetpackage =  'LOAD DATA TABLE TRANSFORMATIONMODEL  \n'
    	              targetpackage = targetpackage + 'OWNER                 1 DEFAULT      NULL TABLENAME            2 DEFAULT      NULL \nDATASTREAMNAME          3 DEFAULT      NULL \n'
                      targetpackage = targetpackage + 'DATASTREAMHEADERBLOCK 4 DEFAULT      NULL DATASTREAMHEADERPOS  5 DEFAULT      NULL \nDATASTREAMTRAILERBLOCK  6 DEFAULT      NULL DATASTREAMTRAILERPOS  7 DEFAULT      NULL  DATASTREAMDATABLOCK  8 DEFAULT      NULL DATASTREAMDATAPOS    9  DEFAULT    NULL \n'
                      targetpackage = targetpackage + '\"EXCLUDE\"          10 DEFAULT      NULL \n'
                      targetpackage = targetpackage + 'PACKAGEGUID          11 DEFAULT      NULL TASKID              12 DEFAULT      NULL \nSTEPID                 13 DEFAULT      NULL SERVERNODE           14 DEFAULT      NULL \n'
                      targetpackage = targetpackage + '\"SERVERDB\"         15 DEFAULT      NULL USERNAME            16 DEFAULT      NULL \n\"CODETYPE\"           17 DEFAULT      NULL CODEPAGEID           18 DEFAULT      NULL \n'
                      targetpackage = targetpackage + '\"STORAGEDEVICE\"    19 DEFAULT      NULL TRANSACTIONSIZE     20 DEFAULT      NULL TRANSFORMATION           21 DEFAULT      NULL \nPART               22 DEFAULT      NULL TABLETYPE            23 DEFAULT      NULL \n'
                      targetpackage = targetpackage + 'CATALOGSTREAMNAME    24 DEFAULT      NULL CATALOGSTREAMTYPE   25 DEFAULT      NULL \nCATALOGSTREAMEBID      26 DEFAULT      NULL CATALOGSTREAMFORMAT  27 DEFAULT      NULL DATASTREAMTYPE       28 DEFAULT      NULL  DATASTREAMEBID     29 DEFAULT     NULL \n'
                      targetpackage = targetpackage + 'DATASTREAMFORMAT     30 DEFAULT      NULL \"RESTART\"         31 DEFAULT      NULL \nRESTORE                32 DEFAULT      NULL PAGECOUNT            33 DEFAULT      NULL \n'
                      targetpackage = targetpackage + 'ROWCOUNT             34 DEFAULT      NULL STARTTIME           35 DEFAULT      NULL \nENDTIME                36 DEFAULT      NULL REQUESTTIME          37 DEFAULT      NULL \n'
                      targetpackage = targetpackage + 'RECEIVETIME          38 DEFAULT      NULL IOTIME              39 DEFAULT      NULL \nLOGTIME                40 DEFAULT      NULL '
                      targetpackage = targetpackage + " INSTREAM FILE '%s'  COMPRESSED %s \n" % (os.path.join(self.options.targetindepdatapath,  'wrk', '%s' % self.options.targetdbname , 'LOADER', '%s' % (file)), self.options.streamcharencoding)

                      print 'Load      Transformation Package x\'%s\' of LOAD    TRANSFORMATION' %  self.targetloader.packageguid
                      try :
 	                  self.targetloader.loadersession.cmd ("%s" %  targetpackage )
	              except loader.LoaderError, err:
    	                  print 'command failed:' , err

    	      cmd = '  SELECT T1.PACKAGEGUID , T1.TRANSFORMATION, T1.OWNER, T1.TABLENAME , T1.DATASTREAMHEADERBLOCK, T1.DATASTREAMHEADERPOS , T1.DATASTREAMDATABLOCK, T1.DATASTREAMDATAPOS , T1.START '
    	      cmd = cmd + '   FROM   TRANSFORMATIONSTEPS T1, TRANSFORMATIONSTEPS T2 '
	      cmd = cmd + '   WHERE  ( T1.start  <> T2.start  or T1.datastreamheaderpos <>  T2.datastreamheaderpos or  T1.datastreamdatapos <> T2.datastreamdatapos  ) '
              cmd = cmd + '   AND T1.owner = T2.owner and T1.tablename = T2.tablename '


	      self.cmd = cmd
	      try :
 	          self.targetloader.loadersession.cmd ("%s" % self.cmd  )
	      except loader.LoaderError, err:
	          if err.sqlCode == 100:
	             print 'OK'
	          else:
    	             print 'command failed:' , err

      def cmd_purify_package(self):
          if  socket.gethostname() == 'p30882' :  #p30882
              self.sourceloader.loadersession.release()
              self.targetloader.loadersession.release()
              try :
                   os.chdir (os.path.join (os.environ ['INSTROOT'], 'wrk'))
              except OSError, (errno, err ):
                   print 'command failed: ', err

              #source
              if  os.path.isfile      ( self.sourceloader.protocol ) :
                  transformationstep = []
                  protocol      = open( self.sourceloader.protocol ,'r')
                  protocolline  = protocol.readline()
                  while protocolline :
                        match_1 = re.search ('// R', protocolline )
                        if match_1 != None :
                           cmd  = string.replace( protocolline , "// R\t" , '//\n')
                           transformationstep.append(cmd)
                        protocolline  = protocol.readline()
                  protocol.close()
                  time.sleep(1)
                  protocol      = open( self.sourceloader.protocol ,'a+')
                  protocol.writelines(transformationstep)
                  protocol.close()

                  sourceloader_batch  = string.replace(self.sourceloader.protocol, 'prt', 'bat' )
                  sourceloader_purify = string.replace(self.sourceloader.protocol, 'prt', 'pur' )
                  try:
                     os.rename(self.sourceloader.protocol, sourceloader_batch )
                  except OSError, err:
                     print 'command failed:', err
                  purify_loadercli  = '\n%s -b %s -E 10\n'   % (os.path.join(self.options.sourceindepdatapath,'bin','loadercli'), sourceloader_batch)
                  #purify_loadercli  = 'purify /SaveTextData=%s /ExcludeSystemDir %s.exe -b %s'   % (sourceloader_purify,os.path.join( self.options.sourceindepdatapath,'bin','loadercli'), sourceloader_batch)
                  print purify_loadercli
                  try:
                      os.system (purify_loadercli)
                  except OSError, err:
                      print 'command failed:', err

              #target
              if  os.path.isfile      ( self.targetloader.protocol ) :
                  transformationstep = []
                  protocol      = open( self.targetloader.protocol ,'r')
                  protocolline  = protocol.readline()
                  while protocolline :
                        match_1 = re.search ('// R', protocolline )
                        if match_1 != None :
                           cmd  = string.replace( protocolline , "// R\t" , '//\n')
                           transformationstep.append(cmd)
                        protocolline  = protocol.readline()
                  protocol.close()
                  time.sleep(1)
                  protocol      = open( self.targetloader.protocol ,'a+')
                  protocol.writelines(transformationstep)
                  protocol.close()

                  targetloader_batch  = string.replace(self.targetloader.protocol, 'prt', 'bat' )
                  targetloader_purify = string.replace(self.targetloader.protocol, 'prt', 'pur' )
                  try:
                     os.rename(self.targetloader.protocol, targetloader_batch )
                  except OSError, err:
                     print 'command failed:', err
                  purify_loadercli  = '\n%s -b %s -E 10\n'   % (os.path.join(self.options.targetindepdatapath,'bin','loadercli'), targetloader_batch)
                  #purify_loadercli  = 'purify /SaveTextData=%s /ExcludeSystemDir %s.exe -b %s'   % (targetloader_purify,os.path.join(self.options.targetindepdatapath,'bin','loadercli'),targetloader_batch)
                  print purify_loadercli
                  try:
                      os.system (purify_loadercli)
                  except OSError, err:
                      print 'command failed:', err

class Session:
      def __init__( self, options = None ) :
          self.options       = options
          self.sourcesession = None
          self.sql           = None
          self.cmd           = None
          self.result        = None

          # sapdb session
  	  try:
              self.sourcesession = sql.connect ( regsub.split(self.options.sourcedbauser,',')[0], regsub.split(self.options.sourcedbauser,',')[1], self.options.sourcedbname, self.options.sourcenode )
          except sql.SQLError, err :
              print 'command failed:' , err

      def cmd_execute(self):
          try:
              self.result = self.sourcesession.sql(self.cmd)
              self.result = self.result.next()
          except sapdb.SQLError,  err :
              print 'sql command failed:', err

class StreamLoader :
      def __init__(self, sapdb_options         = None, sapdbloader_version  = None, sapdb_node     = None, sapdb_         = None, sapdb_user   = None, sapdb_pwd      = None, sapdb_catalog  = None,
                         stream_transformation = None, stream_medium        = None, stream_format  = None, stream_coding  = None, stream_blob  = None, loader_restart = None  ):

          self.options                = sapdb_options
          self.sapdbnode              = sapdb_node
          self.sapdbroot              = sapdbloader_version
          self.sapdb                  = sapdb_
          self.user                   = sapdb_user
          self.pwd                    = sapdb_pwd
          self.catalog                = string.upper(sapdb_catalog)
          self.package                = None
          self.packagecode            = 'ASCII'
          self.packageguid            = None
          self.packageload            = 'FALSE'
          self.packageunload          = 'FALSE'
          self.protocol               = None
          self.streamtransformation   = string.upper(stream_transformation)
          self.streammedium           = string.upper(stream_medium )
          self.streamformat           = string.upper(stream_format )
          self.streamcharencoding     = string.upper(stream_coding )
          if stream_blob == 'SINGLE':
             self.blobmask            = ''
          if stream_blob == 'MULTIPLE':
             self.blobmask            = '#######'
          self.restart                = string.upper(loader_restart)
          self.cmd                    = None
          # medium file
          self.ddlfile                = os.path.join (self.sapdbroot, 'wrk', '%s.catalog'  % self.sapdb )
          self.datfile                = os.path.join (self.sapdbroot, 'wrk', '%s.data'     % self.sapdb )
          # medium pipe
          self.ddlpipe                = "\\\\.\PIPE\\catalog"
          self.datpipe                = "\\\\.\PIPE\\data"
          self.datpipe_source         = "\\\\.\PIPE\\source.data"
          self.datpipe_target         = "\\\\.\PIPE\\target.data"
          self.ddlpipe_source         = "\\\\.\PIPE\\source.catalog"
          self.ddlpipe_target         = "\\\\.\PIPE\\target.catalog"
          # medium tape
          self.ddlpipe_ebid           = None
          self.datpipe_ebid           = None
          self.mappipe_ebid           = None
          # statement
          self.cmd

          # create sapdb loader session
          try:
              self.loadersession =  loader.Loader(self.sapdbnode, '' ,self.sapdbroot )
              #version
              self.cmd = 'VERSION'
              version = self.cmd_execute()
              #protocol odbc
              self.cmd = 'SET ODBCTRACE        OFF '
              self.cmd_execute()
              #protocol loader
              self.cmd = 'SET'
              configuration = self.cmd_execute()
              self.protocol = string.strip(string.replace(string.replace(configuration,'Loader Server Protocol File:',''),'\'',''))
              print 'PROTOCOLFILE  %s ' % self.protocol
              #sqlmode
              self.cmd = 'SQLMODE INTERNAL'
              self.cmd_execute()
              #transaction
              self.cmd = 'AUTOCOMMIT ON'
              self.cmd_execute()
              self.cmd = 'SET TRANSACTION SIZE 1000'
              self.cmd_execute()
              self.cmd = 'SET MAXERRORCOUNT    100'
              self.cmd_execute()
              #format
              self.cmd = 'SET COMPRESSED       \'/,/\"/\''
              self.cmd_execute()
              if self.options.demo == 'TRUE':
                 print 'SET BLOBMASK        %s'   % self.blobmask
              self.cmd = 'SET LANGUAGE         \'ENG\' '
              self.cmd_execute()
              #datatypes
              self.cmd = 'SET CODETYPE         ASCII '
              self.cmd_execute()
              self.cmd = 'SET CODEPAGE         ISO-8859-1 '
              self.cmd_execute()
              self.cmd = 'SET BOOLEAN          \'TRUE/FALSE\' '
              self.cmd_execute()
              self.cmd = 'SET DECIMAL          \'//./\''
              self.cmd_execute()
              self.cmd = 'SET NULL             \'?\''
              self.cmd_execute()
              self.cmd = 'SET DATE             ISO'
              self.cmd_execute()
              self.cmd = 'SET TIME             ISO'
              self.cmd_execute()
              self.cmd = 'SET TIMESTAMP        ISO'
              self.cmd_execute()
              #medium
              self.cmd = 'SET BACKUPTOOL       NETWORKER '
              self.cmd_execute()
              if self.sapdbroot <> self.sapdb :
                 self.cmd_use_serverdb_user()

          except loader.CommunicationError, err :
                 print 'loader communication failed:', err
          except loader.LoaderError, err:
                 print 'command failed:' , err

    	  # create sapdb session
    	  try:
             self.sapdbsession = sql.connect ( self.user, self.pwd, self.sapdb, self.sapdbnode )
          except sql.SQLError, err :
             print 'command failed:' , err

      def cmd_execute(self):
          rc  = 0
          cmd = self.cmd
          cmd = string.replace(cmd,os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.options.sourcedbname , 'LOADER/' ),'')
          cmd = string.replace(cmd,os.path.join(self.options.targetindepdatapath,  'wrk', '%s' % self.options.targetdbname , 'LOADER/' ),'')
          cmd = string.replace(cmd,os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.options.sourcedbname , 'LOADER\\'),'')
          cmd = string.replace(cmd,os.path.join(self.options.targetindepdatapath,  'wrk', '%s' % self.options.targetdbname , 'LOADER\\'),'')

 	  if socket.gethostname() == 'p30882' : #p30882
 	     self.packageload   = 'TRUE'
 	     self.packageunload = 'TRUE'
 	     cmd = self.cmd
 	     cmd = string.replace(cmd,'"' ,'')
 	     cmd = string.replace(cmd,'\n','')
 	     cmd = "REMARK \"%s\"" % cmd
 	     if self.options.demo == 'TRUE':
                print '//\n%s' % cmd
                pass
 	     try :
    	         self.loadersession.cmd ("%s" %cmd )
    	     except loader.LoaderError, err:
 	         print 'command failed: ', err

 	     if mutex.locked() == 1:
                mutex.release()    #release

 	  else:
              if cmd[0:3] == 'COM' or cmd[0:3] == 'DEL' or cmd[0:3] == 'SET' or cmd[0:3] == 'AUT' or cmd[0:3] == 'SQL' or cmd[0:3] == 'VER':
                 print '%s' % cmd
              else:
                 print '//\n%s' % cmd
 	      try :
 	          rc = self.loadersession.cmd ("%s" % self.cmd )
	      except loader.LoaderError, err:
	          if(err.sqlCode   ==  100   or
	             err.sqlCode   == -4004  or
	             err.sqlCode   == -5001  or
	             err.sqlCode   == -6000  or
	             err.sqlCode   == -6008  or
	             err.sqlCode   == -7017  or
	             err.errorCode == -25362  ):
                     pass
	          else:
 	             print 'command failed: ', err
 	  self.cmd = ''
 	  return rc

      def cmd_use_serverdb_user(self):
	  cmd          = 'USE USER %s %s SERVERDB %s ON %s ' % ( self.user, self.pwd, self.sapdb, self.sapdbnode )
	  self.cmd     = cmd
	  self.cmd_execute()

      def cmd_export_catalog_package(self):
          #Medium
          if   self.streammedium[0:4] == 'PIPE' :
               streammedium = 'PIPE'
          else :
               streammedium = self.streammedium

          #Format
          if( self.streamformat == 'DDL' or
              self.streamformat == 'XML'    ):
              #Medium
              if self.options.streammedium == 'FILE' :
                 cmd = "EXTRACT CATALOG ALL"
                 cmd = cmd +   " OUTSTREAM FILE '%s' %s          "   % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s.catalog' % (self.options.sourcedbname )),self.options.streamformat,)
              #Medium
              if(self.options.streammedium == 'TAPE' or
                 self.options.streammedium == 'PIPE'   ):
                 cmd = 'EXTRACT CATALOG ALL \n'
                 cmd = cmd +   " OUTSTREAM  %s '%s'  %s    %s    "   % (streammedium, self.ddlpipe_source, self.options.streamformat, self.streamcharencoding )

              if self.streamformat  == 'XML':
                 self.packageload    = 'TRUE'
                 self.packageunload  = 'TRUE'
                 mutex.release()

          #Format
          if( self.streamformat == 'PAGES'   or
              self.streamformat == 'RECORDS'    ):
              restartRecord = self.cmd_restart_package()
              #Medium
              if self.options.streammedium == 'FILE' :
                 cmd = 'EXTRACT DB \n'
                 cmd = cmd +   " CONFIGURATION  INSTREAM  FILE '%s' COMPRESSED  %s     \n"  % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s.configuration' % self.sapdb ), self.options.streamcharencoding )
                 cmd = cmd +   " CATALOG        OUTSTREAM FILE '%s' DDL         %s     \n"  % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s.catalog'       % self.sapdb ), self.streamcharencoding )
	         cmd = cmd +   " DATA           OUTSTREAM FILE '%s' %s          %s     \n"  % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s.data'          % self.sapdb ), self.options.streamformat, self.streamcharencoding )
 	         cmd = cmd +   " PACKAGE        OUTSTREAM FILE '%s' COMPRESSED  %s       "  % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s.package'       % self.sapdb ), self.streamcharencoding )

              #Medium
              if(self.options.streammedium == 'TAPE' or
                 self.options.streammedium == 'PIPE'   ):
                 cmd = 'EXTRACT DB \n'
                 cmd = cmd +   " CONFIGURATION  INSTREAM  FILE '%s'  COMPRESSED %s     \n"  % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s.configuration' % self.sapdb ), self.streamcharencoding )
                 cmd = cmd +   " CATALOG        OUTSTREAM %s   '%s'  DDL        %s     \n"  % (streammedium, self.ddlpipe_source                     , self.streamcharencoding )
                 cmd = cmd +   " DATA           OUTSTREAM %s   '%s'  %s         %s     \n"  % (streammedium, self.datpipe_source , self.streamformat , self.streamcharencoding )
                 cmd = cmd +   " PACKAGE        OUTSTREAM FILE '%s'  COMPRESSED %s       "  % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s.package'       % self.sapdb ), self.streamcharencoding )
              #Start
              if restartRecord <> [] :
                 cmd = cmd + " RESTART '%s' " % (restartRecord[0])

          self.cmd     = cmd
          self.cmd_execute()

      def cmd_import_catalog_package(self):
          #Medium
          if  self.streammedium      == 'TAPE' :
              self.ddlpipe_target =  self.ddlpipe_source
              self.datpipe_source =  self.datpipe_source
          if  self.streammedium[0:4] == 'PIPE' :
              streammedium = 'PIPE'
          else:
              streammedium = self.streammedium

          #Encoding
          if self.catalog == 'UNICODE' :
             self.packagecode  = 'UCS2'
          if self.catalog == 'ASCII' :
             self.packagecode  = 'ASCII'

          #Format
          if( self.streamformat == 'DDL' or
              self.streamformat == 'XML'   ):
              cmd = 'LOAD CATALOG ALL '
              #Medium
              if self.streammedium      == 'FILE':
                 cmd = cmd + "                   INSTREAM FILE '%s'             %s          " % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s.catalog' % (self.options.sourcedbname )),self.streamformat)
              #Medium
              if self.streammedium[0:4] == 'PIPE' :
                 cmd = cmd + "                   INSTREAM  %s  '%s'             %s  %s      " % (streammedium, self.ddlpipe_target                       , self.streamformat, self.streamcharencoding   )
              #Medium
              if self.streammedium      == 'TAPE':
                 cmd = cmd + "                   INSTREAM  %s  EBID %s  '%s'    %s  %s      " % (streammedium, self.ddlpipe_ebid , self.ddlpipe_target   , self.streamformat, self.streamcharencoding   )

              if self.streamformat  == 'XML':
                 self.packageload    = 'TRUE'
                 self.packageunload  = 'TRUE'

          #Format
          if( self.streamformat == 'PAGES'   or
              self.streamformat == 'RECORDS'    ):
              restartRecord = self.cmd_restart_package()

              #Medium
              if self.streammedium[0:4] == 'FILE' :
                 cmd = 'LOAD DB \n'
                 cmd = cmd +    " CONFIGURATION  INSTREAM FILE  '%s' COMPRESSED  %s   \n" % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.sapdb , 'LOADER', '%s.configuration' % self.sapdb), self.streamcharencoding )
                 cmd = cmd +    " CATALOG        INSTREAM FILE  '%s' DDL         %s   \n" % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s.catalog'       % self.options.sourcedbname), self.streamcharencoding)
                 cmd = cmd +    " DATA           INSTREAM FILE  '%s' %s          %s   \n" % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s.data'          % self.options.sourcedbname), self.options.streamformat, self.streamcharencoding)
              #Medium
              if self.streammedium[0:4] == 'PIPE' :
                 cmd = 'LOAD DB \n'
                 cmd = cmd +    " CONFIGURATION  INSTREAM FILE  '%s' COMPRESSED  %s   \n" % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.sapdb , 'LOADER', '%s.configuration' % self.sapdb), self.streamcharencoding )
                 cmd = cmd +    " CATALOG        INSTREAM  %s   '%s' DDL         %s   \n" % (streammedium, self.ddlpipe_target                     , self.streamcharencoding   )
                 cmd = cmd +    " DATA           INSTREAM  %s   '%s' %s          %s   \n" % (streammedium, self.datpipe_source , self.streamformat , self.streamcharencoding   )
              #Medium
              if self.streammedium      == 'TAPE':
                 cmd = 'LOAD DB \n'
                 cmd = cmd +    " CONFIGURATION  INSTREAM FILE  '%s' COMPRESSED  %s        \n" % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.sapdb , 'LOADER', '%s.configuration' % self.sapdb), self.streamcharencoding )
                 cmd = cmd +    " CATALOG        INSTREAM  %s   EBID %s  '%s'   DDL   %s   \n" % (streammedium, self.ddlpipe_ebid , self.ddlpipe_target ,                       self.streamcharencoding   )
                 cmd = cmd +    " DATA           INSTREAM  %s   EBID %s  '%s'    %s   %s   \n" % (streammedium, self.mappipe_ebid , self.datpipe_source , self.streamformat   , self.streamcharencoding   )
              cmd = cmd +       " PACKAGE        OUTSTREAM FILE '%s' COMPRESSED  %s          " % (os.path.join(self.options.targetindepdatapath, 'wrk', '%s' % self.sapdb , 'LOADER', '%s.package' % (self.sapdb)), self.packagecode )
              #Start
              if restartRecord <> [] :
                 cmd = cmd +    " RESTART '%s' " % (restartRecord[0])

          if self.streamformat  <> 'XML':
             self.cmd     = cmd
             self.cmd_execute()

      def cmd_export_schema_package(self):
          #Medium
          if   self.streammedium[0:4] == 'PIPE' :
               streammedium = 'PIPE'
          else :
               streammedium = self.streammedium

          #Format
          if( self.streamformat == 'PAGES'   or
              self.streamformat == 'RECORDS'    ):
              cmd = 'EXTRACT TABLE USER \n'
              cmd = cmd +      " CONFIGURATION  INSTREAM  FILE '%s'    COMPRESSED %s \n" % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.sapdb , 'LOADER', '%s.configuration' % (self.sapdb )), self.streamcharencoding )
              #Medium
              if(self.options.streammedium == 'TAPE' or
                 self.options.streammedium == 'PIPE'   ):
                 cmd = cmd +   " DATA           OUTSTREAM %s   '%s'    %s         %s \n" % (streammedium, self.datpipe_source , self.streamformat  , self.streamcharencoding )
                 cmd = cmd +   " PACKAGE        OUTSTREAM FILE '%s'    COMPRESSED %s   " % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.sapdb , 'LOADER', '%s.package'       % (self.sapdb )), self.streamcharencoding )
              #Medium
              if(self.options.streammedium == 'FILE'     ):
   	         cmd = cmd +   " DATA           OUTSTREAM FILE '%s'    %s         %s \n" % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s_%s.data'       % (self.options.sourcedbname,self.user )),self.options.streamformat,self.streamcharencoding )
 	         cmd = cmd +   " PACKAGE        OUTSTREAM FILE '%s'    COMPRESSED %s   " % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s_%s.package'    % (self.options.sourcedbname,self.user)), self.streamcharencoding )

          #Format
          if( self.streamformat == 'DDL' or
              self.streamformat == 'XML'      ):
              cmd = 'EXTRACT CATALOG USER \n'
              #Medium
              if(self.options.streammedium == 'TAPE' or
                 self.options.streammedium == 'PIPE'   ):
                 cmd = cmd +   " OUTSTREAM %s   '%s'  %s   %s  "  % ( streammedium, self.ddlpipe_source, self.options.streamformat, self.streamcharencoding )
              #Medium
              if self.options.streammedium == 'FILE'  :
                 cmd = cmd +   " OUTSTREAM FILE '%s'  %s   %s  "    % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % self.sapdb , 'LOADER', '%s_%s.dtl'  % (self.sapdb,self.user )),self.options.streamformat, self.streamcharencoding)

          self.cmd     = cmd
          self.cmd_execute()

      def cmd_import_schema_package(self):
          #Medium
          if  self.streammedium      == 'TAPE' :
              self.ddlpipe_target =  self.ddlpipe_source
              self.datpipe_source =  self.datpipe_source
          if  self.streammedium[0:4] == 'PIPE' :
              streammedium = 'PIPE'
          else:
              streammedium = self.streammedium

          if self.catalog == 'UNICODE' :
             self.packagecode  = 'UCS2'
          if self.catalog == 'ASCII' :
             self.packagecode  = 'ASCII'

          #Format
          if( self.streamformat == 'PAGES'   or
              self.streamformat == 'RECORDS'    ):
              cmd = 'LOAD TABLE    USER \n'
              cmd =       cmd + " CONFIGURATION INSTREAM  FILE '%s' COMPRESSED %s \n" % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.sapdb , 'LOADER', '%s.configuration' % (self.sapdb )), self.streamcharencoding )
              #Medium
              if self.streammedium[0:4] == 'PIPE':
                 cmd =    cmd + " DATA          INSTREAM  %s   '%s' %s         %s \n" % (streammedium, self.datpipe_source , self.streamformat , self.streamcharencoding   )
                 cmd =    cmd + " PACKAGE       OUTSTREAM FILE '%s' COMPRESSED %s   " % (os.path.join(self.options.targetindepdatapath, 'wrk', '%s' % self.sapdb , 'LOADER', '%s.package' % (self.sapdb)), self.packagecode )
              #Medium
              if self.streammedium      == 'TAPE':
                 cmd =    cmd + " DATA          INSTREAM  %s   '%s' EBID %s %s %s \n" % (streammedium, self.datpipe_source , self.mappipe_ebid , self.streamformat , self.streamcharencoding   )
                 cmd =    cmd + " PACKAGE       OUTSTREAM FILE '%s' COMPRESSED %s   " % (os.path.join(self.options.targetindepdatapath, 'wrk', '%s' % self.sapdb , 'LOADER', '%s.package' % (self.sapdb)), self.packagecode )
              #Medium
              if self.streammedium      == 'FILE':
                 if self.options.streamformat   == 'PAGES'   :
 	            cmd = cmd + " DATA          INSTREAM  FILE '%s' %s            \n" % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s_%s.data'       % (self.options.sourcedbname,self.user )),self.options.streamformat)
      	            cmd = cmd + " PACKAGE       OUTSTREAM FILE '%s' COMPRESSED %s   " % (os.path.join(self.options.targetindepdatapath, 'wrk', '%s' % self.options.targetdbname , 'LOADER', '%s_%s.package'    % (self.options.targetdbname,self.user)), self.packagecode)
 	         if self.options.streamformat   == 'RECORDS' :
 	            cmd = cmd + " DATA          INSTREAM  FILE '%s' %s         %s \n" % (os.path.join(self.options.sourceindepdatapath, 'wrk', '%s' % self.options.sourcedbname , 'LOADER', '%s_%s.data'       % (self.options.sourcedbname,self.user )),self.options.streamformat,self.streamcharencoding)
 	            cmd = cmd + " PACKAGE       OUTSTREAM FILE '%s' COMPRESSED %s   " % (os.path.join(self.options.targetindepdatapath, 'wrk', '%s' % self.options.targetdbname , 'LOADER', '%s_%s.package'    % (self.options.targetdbname,self.user)), self.packagecode)

          #Format
          if( self.streamformat == 'DDL' or
              self.streamformat == 'XML'      ):
              cmd = 'LOAD CATALOG USER\n'
              #Medium
              if self.streammedium[0:4] == 'PIPE' :
                 cmd = cmd + "                INSTREAM  %s  '%s'          %s  %s  \n" % (streammedium, self.ddlpipe_target                     , self.options.streamformat  , self.streamcharencoding  )
              #Medium
              if self.streammedium      == 'TAPE':
                 cmd = cmd + "                INSTREAM  %s  '%s' EBID %s  %s  %s  \n" % (streammedium, self.ddlpipe_target , self.ddlpipe_ebid , self.options.streamformat , self.streamcharencoding   )
              #Medium
              if self.streammedium      == 'FILE':
                 cmd = cmd + "                INSTREAM FILE '%s'          %s   %s   " % (os.path.join(self.options.sourceindepdatapath,  'wrk' , '%s' % self.options.sourcedbname , 'LOADER', '%s_%s.dtl'  % (self.options.sourcedbname,self.user )), self.options.streamformat, self.streamcharencoding )

              if self.streamformat  == 'XML':
                 self.packageload    = 'TRUE'
                 self.packageunload  = 'TRUE'

          if self.streamformat  <> 'XML':
             self.cmd     = cmd
             self.cmd_execute()

      def cmd_export_table_package(self):
          sapdb        = self.package.transformation.sourceloader.sapdb
          user         = self.package.user
          table        = self.package.table

          #Medium
          if self.options.streammedium    == 'FILE'  :
             #Format
             if(self.options.streamformat == 'XML' or
                self.options.streamformat == 'DDL' ) :
                cmd = 'EXTRACT CATALOG TABLE '
                cmd = cmd + " \"%s\".\"%s\" \n"                 % (user, table)
                cmd = cmd + "      OUTSTREAM FILE '%s' %s   \n" % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % sapdb , 'LOADER', '%s_%s_%s.dtl'  % (sapdb,user,table )),self.options.streamformat)
                self.cmd = cmd
	        self.cmd_execute()
             #Format
 	     if(self.options.streamformat == 'PAGES' or
 	        self.options.streamformat == 'RECORDS'  ):
                cmd = 'EXTRACT TABLE TABLE '
 	        cmd = cmd + " \"%s\".\"%s\" \n"                 % ( user, table)
 	        cmd = cmd + " DATA OUTSTREAM FILE '%s' %s %s\n" % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % sapdb , 'LOADER', '%s_%s_%s.data' % (sapdb,user,table )),self.options.streamformat,self.options.streamcharencoding)
	        self.cmd = cmd
	        self.cmd_execute()
             #Format
             if(self.options.streamformat[0:9] == 'FORMATTED'  or
                self.options.streamformat      == 'COMPRESSED'    ):
                cmd = 'EXTRACT DATA FOR LOAD %s TABLE '         % (self.options.streamformat_target)
 	        cmd = cmd + " \"%s\".\"%s\" \n"                 % (user, table)
 	        cmd = cmd + "      OUTSTREAM FILE '%s'      \n" % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % sapdb , 'LOADER', '%s_%s_%s.dtl'  % (sapdb,user,table )))
 	        cmd = cmd + "      OUTSTREAM FILE '%s' %s %s\n" % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % sapdb , 'LOADER', '%s_%s_%s.data' % (sapdb,user,table )),self.options.streamformat,self.options.streamcharencoding)
             	cmd = cmd + self.package.statement_long
                self.cmd = cmd
	        self.cmd_execute()

          #Medium
          if(self.options.streammedium[0:4] == 'PIPE' ):
             #Format
             if( self.options.streamformat == 'PAGES' or
                 self.options.streamformat == 'RECORDS'  ) :
                 cmd  = 'EXTRACT TABLE TABLE '
                 cmd  = cmd + " \"%s\".\"%s\" \n"                 % ( user, table )
 	         cmd  = cmd + " DATA OUTSTREAM PIPE '%s' %s %s\n" % ('\\\\.\\PIPE\\%s_%s_%s.dat' % ( sapdb, user, table ), self.options.streamformat, self.options.streamcharencoding )
 	         self.cmd = cmd
 	         self.cmd_execute()
             #Format
             if( self.options.streamformat == 'FORMATTED'  or
                 self.options.streamformat == 'COMPRESSED'      ):
                 cmd  = 'EXTRACT DATA FOR LOAD %s TABLE '       % (self.options.streamformat_target)
                 cmd  = cmd + " \"%s\".\"%s\" \n"               % (user, table)
 	         cmd  = cmd + " OUTSTREAM PIPE '%s'         \n" % ('\\\\.\\PIPE\\%s_%s_%s.dtl' % ( sapdb, user, table ) )
 	         cmd  = cmd + " OUTSTREAM PIPE '%s' %s   %s   " % ('\\\\.\\PIPE\\%s_%s_%s.dat' % ( sapdb, user, table ), self.options.streamformat, self.options.streamcharencoding )
 	         cmd  = cmd + self.package.statement_long
 	         self.cmd = cmd
 	         self.cmd_execute()

      def cmd_import_table_package(self):
          sapdb  = self.package.transformation.sourceloader.sapdb
          user   = self.package.user
          table  = self.package.table

          #Medium
          if self.options.streammedium  == 'FILE'  :
             #Format
             if( self.options.streamformat == 'PAGES'   or
                 self.options.streamformat == 'RECORDS'    ) :
                 cmd = 'LOAD TABLE TABLE '
 	         cmd = cmd + " \"%s\".\"%s\" \n"                % (user, table)
 	         cmd = cmd + " DATA INSTREAM FILE '%s' %s %s\n" % (os.path.join(self.options.sourceindepdatapath,  'wrk', '%s' % sapdb , 'LOADER', '%s_%s_%s.data' % (sapdb,user,table )),self.options.streamformat,self.options.streamcharencoding)
                 self.cmd = cmd
                 self.cmd_execute()
              #Format
             if( self.options.streamformat == 'DDL'        or
                 self.options.streamformat == 'FORMATTED'  or
                 self.options.streamformat == 'COMPRESSED'   ):
                 for cmd in self.package.transformation_step :
                     self.cmd = cmd
                     self.cmd_execute()
                     #grant and execute
                     if string.find(cmd,'Create') == 0 and string.find(cmd,'Table')  == 0 :
                        self.cmd= 'GRANT ALL ON %s TO PUBLIC' % table

          #Medium
          if(self.options.streammedium[0:4] == 'PIPE' ):
             #Format
             if( self.options.streamformat == 'PAGES'   or
                 self.options.streamformat == 'RECORDS') :
                 cmd      = 'LOAD TABLE TABLE '
                 cmd      = cmd + " \"%s\".\"%s\" \n"               % ( user, table )
 	         cmd      = cmd + " DATA INSTREAM PIPE '%s' %s %s " % ('\\\\.\\PIPE\\%s_%s_%s.dat' % ( sapdb, user, table ), self.options.streamformat, self.options.streamcharencoding)
 	         self.cmd = cmd
                 self.cmd_execute()
             #Format
             if( self.options.streamformat == 'DDL'       or
                 self.options.streamformat == 'FORMATTED' or
                 self.options.streamformat == 'COMPRESSED'   ) :
                 for cmd in self.package.transformation_step :
                     self.cmd = cmd
                     self.cmd_execute()
                     #grant and execute
                     if string.find(cmd,'Create') == 0 and string.find(cmd,'Table')  == 0 :
                        self.cmd= 'GRANT ALL ON %s TO PUBLIC' % table

          self.package.transformation_step = []

      def cmd_load_package(self):
          match_1     = None
          match_2     = None
          packageguid = None
          if  self.streamtransformation == 'EXTRACT' or self.streamtransformation == 'LOAD' :
              protocolfile = os.path.join(self.options.targetindepdatapath,  'wrk', self.protocol )
          if  os.path.isfile      ( protocolfile ) :
              protocol      = open( protocolfile ,'r')
              protocolline  = protocol.readline()

              while protocolline :
                    #user
                    if match_1 == None:
                       match_1  = re.search ('USE USER %s' % self.user, protocolline )
                    if match_1 != None:
                       match_1  = 'ok'
                       match_2  = None
                    if match_2 == None:
                       match_2  = re.search ('LOAD TRANSFORMATIONPACKAGE x', protocolline)
                    if match_2 != None:
                       match_2  = 'ok'
                    #package
                    if match_1 == 'ok' and match_2 == 'ok':
                       packageguid       = regsub.split ( protocolline   , '\'')
                       packageguid       = string.replace( packageguid[1] , "\n" , '')
                       self.packageguid  = packageguid
                       self.packageload  = 'TRUE'
                       if self.options.demo == 'TRUE':
                          print "Load      Transformation Package x'%s'" % self.packageguid

                    protocolline = protocol.readline()

      def cmd_unload_package(self):
          match_1      = None
          match_2      = None
          match_3      = None
          packageguid  = None

          try:
              if  'EXTRACT' == self.streamtransformation  or 'LOAD' == self.streamtransformation :
                  protocolfile = os.path.join(self.options.targetindepdatapath,  'wrk', self.protocol )
              if  os.path.isfile  ( protocolfile ) :
                  protocol  = open( protocolfile ,'r')
                  protocolline  = protocol.readline()
                  counter       = 0

                  while protocolline and  self.packageunload == 'FALSE':
                        #user
                        if match_1 == None:
                           match_1  = re.search ('USE USER %s' % self.user, protocolline )
                        #part
                        if match_1 != None and match_2 == None :
                           if self.options.transformationpackage != 'TABLE' :
                              #medium
                              if 'EXTRACT'  == self.streamtransformation and 'TAPE'  == self.streammedium :
                                  #format
                                  if self.streamformat    == 'DDL' :
                                     if protocolline[0:8] == '// M	NST':
                                          if counter == 0:
                                             ddlpipe_ebid = regsub.split  (protocolline,'NST')
                                             ddlpipe_ebid = string.replace(ddlpipe_ebid[1], "\n" , '')
                                             self.ddlpipe_ebid = "\'NST%s\'" % ddlpipe_ebid
                                  if self.streamformat    ==  'RECORDS' or self.streamformat ==  'PAGES':
                                     if protocolline[0:8] == '// M	NST':
                                        if counter == 0:
                                           ddlpipe_ebid = regsub.split  (protocolline,'NST')
                                           ddlpipe_ebid = string.replace(ddlpipe_ebid[1], "\n" , '')
                                           self.ddlpipe_ebid = "\'NST%s\'" % ddlpipe_ebid
                                        if counter == 1:
                                           datpipe_ebid = regsub.split  (protocolline,'NST')
                                           datpipe_ebid = string.replace(datpipe_ebid[1], "\n" , '')
                                           self.datpipe_ebid = "\'NST%s\'" % datpipe_ebid
                                        if counter == 2:
                                           mappipe_ebid = regsub.split  (protocolline,'NST')
                                           mappipe_ebid = string.replace(mappipe_ebid[1], "\n" , '')
                                           self.mappipe_ebid = "\'NST%s\'" % mappipe_ebid
                                        counter  = counter + 1

                              match_2  = re.search ('UNLOAD TRANSFORMATIONPACKAGE x' , protocolline)
                           if self.options.transformationpackage == 'TABLE' :
                              match_2  = re.search ("%s" % (self.package.table) , protocolline)
                        #format
                        if match_1 != None and match_2 != None :
                           if self.streamformat    == 'DDL':
                              match_3  = 'ok'
                           if match_3 == None :
                              match_3  = re.search ("TABLE EXTRACT successfully executed" , protocolline)
                           if match_3 == None :
                              match_3  = re.search ("TABLE LOAD successfully executed"    , protocolline)
                           if match_3 == None :
                              match_3  = re.search ("Sum of extracted records"            , protocolline)
                           if match_3 == None :
                              match_3  = re.search ("Sum of inserted records"             , protocolline)

                        #package
                        if( ( self.options.transformationpackage != 'TABLE' and match_2 != None ) or
                            ( self.options.transformationpackage == 'TABLE' and match_3 != None ) ):

                           self.packageunload = 'TRUE'

                           if self.options.transformationpackage != 'TABLE' :
                              packageguid        = regsub.split  ( protocolline   , "'")
                              packageguid        = string.replace( packageguid[1] , "\n" , '')
                              self.packageguid   = packageguid

                           if not ( 'EXTRACT'  == self.streamtransformation and 'PIPE'  == self.streammedium ):
                              if  mutex.locked()  == 1 :
                                  mutex.release()           #release

                        protocolline = protocol.readline()

          except IOError, (errno, err ):
             pass

      def cmd_restart_package(self):
          restartRecord = []
          if self.restart == 'RESTART' :
             statement     =  " SELECT DISTINCT PACKAGEGUID, OWNER, TABLENAME, \n DATASTREAMHEADERBLOCK, DATASTREAMHEADERPOS, \n DATASTREAMDATABLOCK, DATASTREAMDATAPOS, ROWCOUNT \n"
	     statement     =  statement + "FROM SYSDBA.TRANSFORMATIONMODEL "
	     statement     =  statement + "WHERE RESTART = TRUE "

             result = self.sapdbsession.sql (statement )
             row    = result.next ()
             if row :
                packageguid = ('%02X' * 24) % tuple (map (ord, row[0] ))
                restartRecord.append (packageguid)
                restartRecord.append (row[1])
                restartRecord.append (row[2])
                restartRecord.append (row[3])
                restartRecord.append (row[4])
                restartRecord.append (row[5])
                restartRecord.append (row[6])
                restartRecord.append (row[7])

          return restartRecord

      def cmd_restart_sourcedb_package(self, targetRecord ):
          #Medium Format Start
          if  self.restart == 'RESTART' and self.streammedium == 'PIPE' and self.streamformat == 'RECORDS' and targetRecord <> [] :
              sourceRecord  = self.cmd_restart_package()
              statement     = " UPDATE SYSDBA.TRANSFORMATIONMODEL SET \n"
              statement     = statement + " DATASTREAMHEADERBLOCK = %s, DATASTREAMHEADERPOS=%s, \n DATASTREAMDATABLOCK=%s,DATASTREAMDATAPOS=%s,\n ROWCOUNT=%s " % ( targetRecord[3] , targetRecord[4], targetRecord[5], targetRecord[6], targetRecord[7] )
              statement     = statement + " WHERE PACKAGEGUID= x'%s' \n AND OWNER = '%s' AND TABLENAME = '%s' AND RESTART = TRUE \n"                            % ( sourceRecord[0], targetRecord[1], targetRecord[2])
              print  statement
              self.sapdbsession.sql (statement )
          #Medium Format Start
          if  self.restart == 'RESTART' and self.streammedium == 'PIPE' and self.streamformat == 'PAGES'   and targetRecord <> [] :
              pass
          #Medium Format Start
          if  self.restart == 'RESTART' and self.streammedium == 'FILE' and self.streamformat == 'RECORDS' and targetRecord <> [] :
              pass
          #Medium Format Start
          if  self.restart == 'RESTART' and self.streammedium == 'FILE' and self.streamformat == 'PAGES'   and targetRecord <> [] :
              pass

      def cmd_restart_targetdb_package(self):
          #Medium Format Start
          if  self.restart == 'RESTART' and self.streammedium == 'PIPE' and self.streamformat == 'RECORDS':
    	      restartRecord = self.cmd_restart_package()
          #Medium Format Start
          if  self.restart == 'RESTART' and self.streammedium == 'PIPE' and self.streamformat == 'PAGES'  :
              restartRecord = self.cmd_restart_package()

              statement     = " UPDATE SYSDBA.TRANSFORMATIONMODEL SET \n"
              statement     = statement + " DATASTREAMHEADERBLOCK = 0, DATASTREAMHEADERPOS=0, \n DATASTREAMDATABLOCK=0,DATASTREAMDATAPOS=0,\n ROWCOUNT=0, RESTART = TRUE \n"
              statement     = statement + " WHERE PACKAGEGUID= x'%s' \n AND OWNER = '%s' AND TABLENAME = '%s' \n"  % (restartRecord[0], restartRecord[1], restartRecord[2])
              print statement
              result        = self.sapdbsession.sql (statement)
              statement     = " DELETE %s.%s \n"  % (restartRecord[1], restartRecord[2] )
              print statement
              result        = self.sapdbsession.sql (statement)
          #Medium Format Start
          if  self.restart == 'RESTART' and self.streammedium == 'FILE' and self.streamformat == 'RECORDS':
              restartRecord = self.cmd_restart_package()
          #Medium Format Start
          if  self.restart == 'RESTART' and self.streammedium == 'FILE' and self.streamformat == 'PAGES'  :
              restartRecord = self.cmd_restart_package()

          return restartRecord

class StreamConnector :
      def __init__(self, source_loader = None, target_loader = None ) :
          self.sourceloader   = source_loader
          self.targetloader   = target_loader
          self.user           = self.sourceloader.package.user
          self.table          = self.sourceloader.package.table
          self.longcolumn     = ''
          self.sourceddltime  = None
          self.sourcedattime  = None
          self.ddlpipe_exist  = 1
          self.datpipe_exist  = 1

      def cmd_ddlpipe2ddlfile(self):
          print '\nStart     StreamConnector to Connect \n FILE %s to\n PIPE %s '  % ( self.sourceloader.ddlfile       , self.sourceloader.ddlpipe_source )
          ddlpipe2file    = "pipe2file -d pipe2file -p %s -f %s"                   % ( self.sourceloader.ddlpipe_source, self.sourceloader.ddlfile        )
          try:
             os.system( ddlpipe2file )
          except OSError, err:
             print 'command failed:', err

      def cmd_ddlfile2ddlpipe(self):
          print '\nStart     StreamConnector to Connect \n FILE %s to\n PIPE %s'   % ( self.sourceloader.ddlfile, self.sourceloader.ddlpipe_source )
          ddlfile2pipe    = "pipe2file -d file2pipe -p %s -f %s"                   % ( self.sourceloader.ddlpipe_source, self.sourceloader.ddlfile )
          print ddlfile2pipe
          try:
             os.system( ddlfile2pipe )
          except OSError, err:
             print 'command failed:', err

      def cmd_datpipe2datfile(self):
          datfile = os.path.join (self.sourceloader.sapdbroot, 'wrk', self.sourceloader.package.sapdb,'LOADER', '%s_%s.dat'      % (self.sourceloader.package.user, self.sourceloader.package.table))
          datpipe2file    = "pipe2file -d pipe2file -p %s           -f %s"         % ( self.sourceloader.datpipe, datfile )
          try:
             print 'Start     StreamConnector to Connect \n FILE %s to\n PIPE %s ' % ( self.sourceloader.datpipe, datfile )
             os.system( datpipe2file )
          except OSError, err:
             print 'command failed:', err

      def cmd_longpipe2longfiles(self):
          for longcolumn in self.sourceloader.package.longcolumns:
              self.longcolumn = longcolumn
              thread.start_new_thread( self.cmd_longpipe2longfile,())
              time.sleep(1)

      def cmd_longpipe2longfile(self):
          longpipe  = '\\\\.\\PIPE\\%s_%s_%s_%s.long'                                                                                            % (self.sourceloader.package.sapdb, self.user ,self.table, self.longcolumn )
          longfile  = os.path.join (self.sourceloader.sapdbroot, 'wrk', self.sourceloader.package.sapdb,'LOADER', '%s_%s_%s.long' % (self.user, self.table, self.longcolumn ))
          longpipe2file = "pipe2file -d pipe2file -p %s -f %s" % ( longpipe , longfile )
          try:
              print 'Start     StreamConnector to Connect \n FILE %s to\n PIPE %s ' % ( longfile, longpipe )
              os.system( longpipe2file )
          except OSError, err:
              print 'command failed:', err

      def cmd_datfile2datpipe(self):
          print '\nStart     StreamConnector to Connect\n FILE %s to\n PIPE %s '    % ( self.sourceloader.datfile, self.sourceloader.datpipe )
          datfile2pipe    = "pipe2file -d file2pipe -p %s           -f %s"          % ( self.sourceloader.datpipe, self.sourceloader.datfile )
          try:
             os.system( datfile2pipe )
          except OSError, err:
             print 'command failed:', err

      def cmd_pipe2tape(self):
          counter = 0
          protocolfile = os.path.join(self.sourceloader.sapdbroot,  'wrk',self.sourceloader.protocol)
          if  os.path.isfile     ( protocolfile ) :
              protocol     = open( protocolfile ,'r')

              if self.sourceloader.streamformat <> 'DDL' :
                 protocolline = protocol.readline()
                 while protocolline :
                     if ( protocolline[0:8] == '// M	NST'):
                          if counter == 0:
                             ddlpipe_ebid = regsub.split  (protocolline,'NST')
                             ddlpipe_ebid = string.replace(ddlpipe_ebid[1] , "\n" , '')
                             self.targetloader.ddlpipe_ebid = "\'NST%s\'" % ddlpipe_ebid
                          if counter == 1:
                             datpipe_ebid = regsub.split  (protocolline,'NST')
                             datpipe_ebid = string.replace(datpipe_ebid[1] , "\n" , '')
                             self.targetloader.datpipe_ebid = "\'NST%s\'" % datpipe_ebid
                          counter  = counter + 1
                     protocolline = protocol.readline()
                 if counter == 2:
                    if mutex.locked() == 1:
                       mutex.release() #release

              if self.sourceloader.streamformat == 'DDL' :
                 protocolline = protocol.readline()
                 while protocolline :
                     if ( protocolline[0:8] == '// M	NST'):
                          if counter == 0:
                             ddlpipe_ebid = regsub.split  (protocolline,'NST')
                             ddlpipe_ebid = string.replace(ddlpipe_ebid[1] , "\n" , '')
                             self.targetloader.ddlpipe_ebid = "\'NST%s\'" % ddlpipe_ebid
                          counter  = counter + 1
                     protocolline = protocol.readline()
                 if counter == 1:
                    if mutex.locked() == 1:
                       mutex.release() #release

      def cmd_ddlpipe2ddltape(self):
          print '\nStart     StreamConnector to Connect \n TAPE to\n PIPE %s ' % ( self.sourceloader.ddlpipe_source )
          ddltapesave  = self.sourceloader.sapdbroot + "\\wrk\\" + self.sourceloader.sapdb + ".ddltape_save"
          ddlpipe2tape = "\nsave    -s %s -N %s -l full -LL %s 2> %s"          % ( self.sourceloader.sapdbnode, self.sourceloader.sapdb,
                                                                                   self.sourceloader.ddlpipe_source, ddltapesave )
          try:
             os.system( ddlpipe2tape )
          except OSError, err:
             print 'command failed:', err
          if  os.path.isfile  ( ddltapesave ) :
              tapefile  = open( ddltapesave ,'r')
              tapeline  = tapefile.readline()
              while tapeline  :
                  if ( tapeline[0:19] == 'completed savetime='):
                       savetime = regsub.split (tapeline,'=')
                       savetime = string.replace( savetime[1] , "\n" , '')
                       self.sourceddltime = savetime
                  tapeline = tapefile.readline()

      def cmd_ddltape2ddlpipe(self):
          print '\nStart     StreamConnector to Connect \n TAPE %s to\n PIPE %s' % ( self.sourceddltime, self.sourceloader.ddlpipe_source )
          ddltape2pipe = "\nrecover -q -s %s -a -t %s -c %s -iY %s "             % ( self.sourceloader.sapdbnode, self.sourceddltime,
                                                                                     self.sourceloader.sapdbnode, self.sourceloader.ddlpipe_source )
          try:
              os.system( ddltape2pipe )
          except OSError, err:
              print 'command failed:', err

      def cmd_datpipe2dattape(self):
          print 'Start     StreamConnector to Connect \n TAPE to\n PIPE %s ' %  self.sourceloader.datpipe_source
          dattapesave  = self.sourceloader.sapdbroot + "\\wrk\\" + self.sourceloader.sapdb + ".dattape_save"
          datpipe2tape = "save    -s %s -N %s -l full -LL %s           2> %s"% ( self.sourceloader.sapdbnode, self.sourceloader.sapdb,
                                                                                 self.sourceloader.datpipe_source , dattapesave  )
          try:
             os.system( datpipe2tape )
          except OSError, err:
             print 'command failed:', err
          if  os.path.isfile  ( dattapesave ) :
              tapefile  = open( dattapesave ,'r')
              tapeline  = tapefile.readline()
              while tapeline :
                  if ( tapeline[0:19] == 'completed savetime='):
                       savetime = regsub.split (tapeline,'=')
                       savetime = string.replace( savetime[1] , "\n" , '')
                       self.sourcedattime = savetime
                  tapeline = tapefile.readline()

      def cmd_dattape2datpipe(self):
          print '\nStart     StreamConnector to Connect\n TAPE %s to\n PIPE %s ' % ( self.sourcedattime, self.sourceloader.datpipe_source )
          dattape2pipe = "\nrecover -q -s %s -a -t %s -c %s -iY %s       "       % ( self.sourceloader.sapdbnode, self.sourcedattime,
                                                                                     self.sourceloader.sapdbnode, self.sourceloader.datpipe_source )
          try:
              os.system( dattape2pipe )
          except OSError, err:
              print 'command failed:', err

class StreamFilter :
      def __init__(self, source_loader = None, target_loader = None ) :
          self.sourceloader   = source_loader
          self.targetloader   = target_loader
          self.ddlerror       = 1
          self.dtlerror       = 1
          self.daterror       = 1

      def cmd_dtlsource2dtltarget(self):
          dtlsource    = None
          if self.sourceloader.options.streammedium == 'FILE':
             dtlsource = os.path.join(self.sourceloader.options.sourceindepdatapath,  'wrk', '%s' % self.sourceloader.sapdb , 'LOADER', '%s_%s_%s.dtl'  % (self.sourceloader.sapdb,self.sourceloader.user,self.sourceloader.package.table ))
          if self.sourceloader.options.streammedium == 'PIPE':
             dtlsource = '\\\\.\\PIPE\\%s_%s_%s.dtl'  % (self.sourceloader.sapdb,self.sourceloader.user,self.sourceloader.package.table )
          try :
             self.dtlerror   = 0
             step            = ''
             dtlsourcestream = open(dtlsource, 'r' )
             sourcedtl       = dtlsourcestream.readline()
             self.targetloader.package.transformation_step = []
             while sourcedtl :
                   targetdtl = string.strip(sourcedtl)
	           if ( targetdtl[0:2] <> "//" ) and targetdtl <> '' :
	                string.replace(targetdtl, 'ASCII', self.sourceloader.options.streamcharencoding)
		        step = step + targetdtl + "\n"
  	           if ( targetdtl[0:2] == "//" or targetdtl == '' )  and step <> '' and step <> ' ':
  	                self.targetloader.package.transformation_step.append(step)
                        step= ''
                   sourcedtl = dtlsourcestream.readline()
             dtlsourcestream.close()
          except IOError, (errno, err ):
             self.dtlerror  = 1


      def cmd_ddlsource2ddltarget(self):
          try :
              self.ddlerror   = 0
              ddlpipesource   = open(self.sourceloader.ddlpipe_source, 'rt' )
              ddlpipetarget   = open(self.targetloader.ddlpipe_target, 'wt' )
              sourceddl       = ddlpipesource.readline()
              while sourceddl :
                    targetddl = string.replace(sourceddl, '%s' % string.upper(self.sourceloader.catalog),
	                                                  '%s' % string.upper(self.targetloader.catalog) )
	            ddlpipetarget.write(targetddl)
	            sourceddl = ddlpipesource.readline()
          except IOError, (errno, err ):
              print 'command failed:', errno, err
              self.ddlerror  = 1

      def cmd_datsource2dattarget_bin(self):
          try :
             self.daterror   = 0
             datpipesource   = open(self.sourceloader.datpipe_source, 'rb')
             datpipetarget   = open(self.sourceloader.datpipe_target, 'wb')
             sourcedat       = datpipesource.read(400)
             while sourcedat :
                   targetdat = sourcedat
                   datpipetarget.write(targetdat)
 	           sourcedat = datpipesource.read(400)
          except IOError, (errno, err ):
             print 'command failed:', errno, err
             self.daterror  = 1

      def cmd_datsource2dattarget_txt(self):
          try :
             self.daterror   = 0
             datpipesource   = open(self.sourceloader.datpipe_source, 'rt')
             datpipetarget   = open(self.sourceloader.datpipe_target, 'wt')
             sourcedat       = datpipesource.readline()
             while sourcedat :
                   targetdat = sourcedat
                   datpipetarget.write(targetdat)
 	           sourcedat = datpipesource.read()
          except IOError, (errno, err ):
             print 'command failed: %s, %s \n' %( errno, err )
             self.daterror  = 1

def check_configuration (options):
    rc = 0

    #Use Case
    options.demo                  =  string.upper(options.demo)
    #Transformation Attributes
    options.transformationtype    =  string.upper(options.transformationtype)
    options.transformationpackage =  string.upper(options.transformationpackage)
    options.streammedium          =  string.upper(options.streammedium)
    options.streamformat          =  string.upper(options.streamformat)
    options.streamformat_target   =  string.upper(options.streamformat_target)
    options.streamcharencoding    =  string.upper(options.streamcharencoding)
    options.streamblob            =  string.upper(options.streamblob)
    options.restart               =  string.upper(options.restart)
    #Transformation Resources
    options.migration             =  string.upper(options.migration)

    #Source DB
    sourcedbmsession = None
    try:
        sourcedbmsession = dbm.DBM (options.sourcenode, options.sourcedbname, '',options.sourcedbmuser)
    except dbm.CommunicationError, err :
        print 'Databasemanager for %s %s not found'  %(options.sourcenode,options.sourcedbname)

    try:
        indepdatapath = sourcedbmsession.cmd('dbm_getpath   INDEPDATAPATH')
        options.sourceindepdatapath = regsub.split(indepdatapath,'\n')[0]
    except dbm.DBMServError, err:
        print 'DB Manager command failed %s '  % err

    try:
        instroot = sourcedbmsession.cmd('dbm_version   INSTROOT')
        options.sourceinstroot = regsub.split(instroot,'\n')[0]
    except dbm.DBMServError, err:
        print 'DB Manager command failed %s '  % err

    try:
        charencoding = sourcedbmsession.cmd('param_getvalue DEFAULT_CODE')
        if charencoding[0:5] == 'ASCII':
           options.sourcecharencoding = 'ASCII'
        if charencoding[0:7] == 'UNICODE':
           options.sourcecharencoding = 'UNICODE'
    except dbm.DBMServError, err:
        print 'DB Manager command failed %s '  % err

    #Target DB
    targetdbmsession = None
    try:
        targetdbmsession = dbm.DBM (options.targetnode, options.targetdbname, '',options.targetdbmuser)
    except dbm.CommunicationError, err :
        print 'Databasemanager for %s %s not found'  %(options.targetnode,options.targetdbname)

    try:
        indepdatapath = targetdbmsession.cmd('dbm_getpath   INDEPDATAPATH')
        options.targetindepdatapath = regsub.split(indepdatapath,'\n')[0]
    except dbm.DBMServError, err:
        print 'DB Manager command failed %s '  % err

    try:
        instroot = targetdbmsession.cmd('dbm_version   INSTROOT')
        options.targetinstroot = regsub.split(instroot,'\n')[0]
    except dbm.DBMServError, err:
        print 'DB Manager command failed %s '  % err

    try:
        charencoding = targetdbmsession.cmd('param_getvalue DEFAULT_CODE')
        if charencoding[0:5] == 'ASCII':
           options.targetcharencoding = 'ASCII'
        if charencoding[0:7] == 'UNICODE':
           options.targetcharencoding = 'UNICODE'
    except dbm.DBMServError, err:
        print 'DB Manager command failed %s '  % err


    if ((options.streamcharencoding  == 'UTF8'   or
         options.streamcharencoding  == 'UCS2'   or
         options.streamcharencoding  == 'ASCII' )and rc == 0 )  :
         rc  = 0
    elif rc == 0:
         print 'streamcoding %s not supported' % options.streamcharencoding
         rc  = 0

    if ((options.streamcharencoding  == 'UCS2'  and
         options.sourcecharencoding  == 'ASCII')and rc == 0  )  :
         print '\n!!Conversion from ASCII data to UNICODE data !! \n This conversion may change information if ASCII is not encoded in CODEPAGE 8859-1\n'
         rc  = 0
    elif rc == 0:
         rc  = 0

    if ((options.streamcharencoding == 'UCS2'  and
         options.targetcharencoding == 'ASCII')and rc == 0  )  :
         print 'Conversion from UNICODE data to ASCII data is not supported; \n this conversion may lose information '
         rc  = 1
    elif rc == 0:
         rc  = 0

    if ((options.streamcharencoding <> ''  and
         options.streamformat == 'PAGES' ) and rc == 0 )  :
         print 'streamcoding %s in streamformat %s not supported' % (options.streamcharencoding, options.streamformat)
         rc  = 0
    elif rc == 0:
         rc  = 0

    if ((options.sourcecharencoding <>
         options.targetcharencoding        and
         options.streamformat == 'PAGES' ) and rc == 0 )  :
         print 'Encoding type of source and target database do not match : source = %s, target = %s. ' % (options.sourcecharencoding, options.targetcharencoding)
         rc  = 1
    elif rc == 0 :
         rc  = 0

    if ((options.streamcharencoding == 'UTF8'  and
       ( options.streamformat == 'PAGES'       or
         options.streamformat == 'RECORDS' ))  and rc == 0 )  :
         print 'streamcoding %s in streamformat %s not supported' % (options.streamcharencoding, options.streamformat)
         rc  = 0
    elif rc == 0:
         rc  = 0

    return rc

if __name__ == '__main__':
   optlib.optMain2 (main, _options () )

